
# Common definitions for Pathena Client are collected here.  These are
# imported by all client modules, directly or indirectly.

import os, sys, time, string, shutil, re

from Tkinter import *
from tkFileDialog import askopenfilename, asksaveasfilename
from tkMessageBox import \
     askokcancel, askyesno, askquestion, showinfo, showwarning, showerror

from app_globals       import *

from client_messages   import *
from logging_services  import *

from update_config     import update_user_prefs

import exceptions
class DeletedFileError(exceptions.Exception):  pass


# ---------- Global constants ---------- #

temp_file = new_temp_file_name()

root = Tk()        # start Tk and save resulting object
root.withdraw()
#root.wm_iconify()
mainwindow = root

client_main_log  = 'pytk_main_log'
client_error_log = 'pytk_error_log'

output_file   = os.path.join(log_dir, client_main_log)
error_file    = os.path.join(log_dir, client_error_log)

open(output_file, 'w').close()           # erase log file before reopening
main_log_file = open(output_file, 'a+')  #   it in append mode
sys.stderr    = open(error_file, 'w')


# When the client is run for the first time, it needs to copy files
# that are subject to user editing.

try:
    prefs_file = os.path.join(config_dir, 'preferences.conf')
    _orig_prefs = os.path.join(config_dir, 'orig_prefs.conf')
    if not os.path.exists(prefs_file):
        shutil.copy(_orig_prefs, prefs_file)
    update_user_prefs(prefs_file, _orig_prefs, app_version)
    preference_load_error = 0
except:
    preference_load_error = 1   ### return failed value ###

_orig_custom_source = os.path.join(types_dir, 'source_custom.orig')
_config_custom_source = os.path.join(config_dir, 'source_custom.py')
_loadable_custom_source = os.path.join(types_dir, 'source_custom.py')
if not os.path.exists(_config_custom_source):
    shutil.copy2(_orig_custom_source, _config_custom_source)
shutil.copy2(_config_custom_source, _loadable_custom_source)

# Following import needs to follow preference update actions.

from preference_utilities import user_prefs
from platform_utilities   import *


# ------------ Color definitions ------------ #

colors = {'master_bg':       '#d7dee4',       # AliceBlue @ 228     ###
          'word_bg_color':   'AliceBlue',
          'text_bg_color':   'ghost white',
          'text_fg_color':   'black',
          'text_bg_gray':    '#f4f4f4',
          'highlight_color': 'DodgerBlue2',   # for entry widgets   ###
          'balloon_color':   '#ffffa0',
          'muted_red':       '#800000',
          'gray_text':       '#909090',
          'gray_red':        '#c0a0a0',

          'headline_alt':    '#f0f0ff',       # pale bluish gray
          'headline_select': '#fffad0',       # pale yellow
          'highlight_line':  '#e8e8f8',       # bluish gray
          'term_occ_fg':     'red',
          'item_meta_fg':    'dark blue',
          'find_text_color': '#e000ff',       # bluish magenta
          'find_error_bg':   '#ffe0e0',       # pale pink
          'headline_occ_fg': '#e000ff',       # bluish magenta
}


# ------------ Font definitions ------------ #

import tkFont

fixed_fonts = ('courier',)
sans_fonts  = ('helvetica',)  ### 'aplos'
serif_fonts = ('new century schoolbook', 'times')

_all_families = tkFont.families()

def find_font(candidates):
    global font_load_error
    family_name = None
    for font in candidates:
        if font in _all_families:
            family_name = font
            break
    if family_name: font_load_error = None
    else:           font_load_error = candidates
    ### to use different base for each family need to adjust formula
    return family_name

font_family = {'fixed': find_font(fixed_fonts),
               'sans':  find_font(sans_fonts),
               'serif': find_font(serif_fonts) }

def font_var(family, size='base', *options):
    family_name = font_family[family]
    try:
        size_value = user_prefs.getint('Client', '%s_%s_size' % (size,family))
    except:
        size_value = 11
    return (family_name, size_value) + options

text_font           = font_var('sans')
serif_text_font     = font_var('serif')
list_font           = font_var('sans')
bold_font           = font_var('sans', 'base', 'bold')
heading_font        = font_var('sans', 'large', 'bold')
title_font          = font_var('sans', 'small', 'bold')
headline_font       = font_var('sans', 'small')
fixed_text_font     = font_var('fixed')
small_text_font     = font_var('sans', 'small')
small_fixed_font    = font_var('fixed', 'small')
small_button_font   = font_var('sans', 'small', 'bold')
italic_font         = font_var('sans', 'base', 'italic')
help_text_font      = font_var('serif')

spacer_font         = (font_family['fixed'], 6)

fixed_text_pixels = tkFont.Font(root=root, font=fixed_text_font).measure('a')


# Unicode shape characters

u_triagup = u'\u25b4'  # black up-pointing small triangle
u_triagrt = u'\u25b8'  # black right-pointing small triangle
u_triagdn = u'\u25be'  # black down-pointing small triangle
u_triaglf = u'\u25c2'  # black left-pointing small triangle
u_wtrigup = u'\u25b5'  # white up-pointing small triangle
u_wtrigdn = u'\u25bf'  # white down-pointing small triangle

dir_indicator = (u_triaglf, '', u_triagrt)   # for +/- 1 indexing


# ---------- Class definitions ---------- #
    
# EventHandler is a wrapper for procedures that are bound to Tkinter events.
# It provides for catching and logging of unhandled exceptions by writing a
# traceback to the debug log.  It optionally records execution times.
# It may also be used in other contexts such as thread procedures
# for the Tkinter 'after' method.

class EventHandler(object):
    def __init__(self, name, proc, show_watch_cursor=root):
        self.name = name
        self.proc = proc
        self.show_watch_cursor = show_watch_cursor

    def record_exception(self):
        emit_log_entry(sys.stderr,
                       "\n***** Handler '%s' raised an exception. *****\n"
                       % self.name)
        trace = StringIO()
        print_exc(100, trace)
        internal_error(trace.getvalue(), 'Event handler error')

    if collect_time_data:
        def __call__(self, *args, **kargs):
            if self.show_watch_cursor:
                self.show_watch_cursor.config(cursor='watch')
                self.show_watch_cursor.update_idletasks()
            start = time.time()
            try:
                self.proc(*args, **kargs)
            except FalseStart:       # these are safely rejected user actions
                pass
            except:                  # unhandled exceptions land here
                self.record_exception()
            if self.proc.__name__ not in exclude_time_data:
                emit_log_entry(main_log_file,
                               "handler '%s'" % self.name, start, time.time())
            if self.show_watch_cursor: self.show_watch_cursor.config(cursor='')
    else:
        def __call__(self, *args, **kargs):
            if self.show_watch_cursor:
                self.show_watch_cursor.config(cursor='watch')
                self.show_watch_cursor.update_idletasks()
            try:
                self.proc(*args, **kargs)
            except FalseStart:       # these are safely rejected user actions
                pass
            except:                  # unhandled exceptions land here
                self.record_exception()
            if self.show_watch_cursor: self.show_watch_cursor.config(cursor='')


# ---------- Utility procedures ---------- #

# Invoking an os command with logging.  Send process output, errors to
# log file according to value of arg redirect:
# 2: redirect stdout, stderr; 1: redirect only stderr; 0: redirect neither

def os_system(cmd, redirect=2, background=0):
    start = time.time()
    if background: bgnd = ' &'
    else:          bgnd = ''
    redir = ('', ' 2>> %s', ' >> %s 2>> %s')[redirect] % \
            ((), (output_file, ), (output_file, output_file))[redirect]
    log_insert = ('\n-------- command begins --------',
                  '\n-------- command stderr output follows --------',
                  '\n-------- command stdout, stderr output follows --------',
                  )[redirect]
    logged_cmd = '%s%s%s' % (cmd, redir, bgnd)
    emit_log_entry(ops_log_file,
                   'Invoking OS command:\n>>> %s%s' % (logged_cmd, log_insert))
    exit_code = os.system(logged_cmd)
    print >> ops_log_file, '-------- command ends --------'
    emit_log_entry(ops_log_file,
                   'Command completed with exit code %s.' % exit_code,
                   start, time.time())
    return exit_code

if collect_time_data:
    def user_dialog(dialog_proc, *args, **nargs):
        start = time.time()
        result = dialog_proc(*args, **nargs)
        emit_log_entry(main_log_file,
                       'User dialog: %s' % dialog_proc.__name__,
                       start, time.time())
        return result
else:
    def user_dialog(dialog_proc, *args, **nargs):
        return dialog_proc(*args, **nargs)

error_dialog_fn = lambda *args: user_dialog(showerror, *args)


# ------------ Error handling ------------ #

def raise_error(title, message):
    user_dialog(showerror, title, message)
    raise UserError

# Extract title, user message and apply string substitution st.

def user_msg_sub(msg_dict, key, st):  ## st is a string tuple or single string
    title, message = msg_dict[key]
    return title, message % st


def eval_assertion(assertion, label, *args):
    if check_assertions:
        start = time.time()
        outcome = assertion()  # Non-false if assertion finds a problem
        if outcome:
            message = "*** Assertion '%s' failed: %s; %s" \
                      % (label, outcome, args)
        else:
            message = "checking assertion '%s'" % label
        stop = time.time()
        emit_log_entry(main_log_file, message, start, stop)
        if outcome: assert 0, 'See log file'

def emit_log_header(label, file):
    bar = '===================================' [:max(0,(70-len(label))/2)]
    print >> file, '\n%s %s %s\n' % (bar, label, bar)


_internal_error_count = 0
_error_file_name = ''
_upload_path = ''

# An internal error causes a log file to be captured and saved.  Then a
# dialog message is displayed for the user.

def internal_error(log_string, title='', show=1):
    global _internal_error_count, _error_file_name, _upload_path
    _internal_error_count += 1
    if _internal_error_count > 5: return     ### stop runaway errors
    # unhandled exceptions in lines below may invoke this function recursively
    emit_log_entry(sys.stderr, '\n-------------------------------\n')
    debug_log(sys.stderr, log_string)
    debug_log(sys.stderr, '-------------------------------\n')
    _error_file_name = time.strftime('err-%H%M-%d%b%y', time.localtime())
    try:
        debug_file_name = os.path.join(log_dir, _error_file_name)
        debug_file = open(debug_file_name, 'w')
        debug_file.write(
            time.strftime('Log file created at %H:%M on %d %b %Y\n\n',
                          time.localtime()))
        debug_file.write('Pathena Client version: %s\n\n' % app_version)
        debug_file.close()

        os_system('cat %s %s >> %s' %
                  (output_file, error_file, debug_file_name), 0)  # no redirect
        _upload_path = debug_file_name
    except:
        pass   # if logging fails, we're too far gone to recover
    if show:
        err_msg = user_dialog(showerror, title,
                              display_messages['internal_error'])
#####        display_reporting_procedure()
    _internal_error_count -= 1

# root.bind('<F1>', display_reporting_procedure)

#def submit_problem_report():
#   internal_error('User Requested Problem Report', show=0)
#   display_reporting_procedure()
   
