
# Dialogs and related processing for running OS commands and
# displaying returned results are collected in this module.


import os, sys, string, glob, thread, Queue
#from time import strftime, localtime, asctime

from file_type_attribs import *

from client_globals import *
from widget_utils   import *


shell_cmd_timeout = user_prefs.get('Client', 'shell_cmd_timeout')


# Start up a terminal window using specified terminal command and user's
# preferred shell.  Accept a string with shell commands to be executed
# before terminal startup.  Optional args to pass to the new shell
# will be appended as blank-delimited strings.

def invoke_terminal(pre_cmds, terminal, exec_option, *args):
    shell = user_prefs.get('System', 'command_shell')
    start_up = '%s %s %s -s' % (terminal, exec_option, shell)
    if pre_cmds:
        term_cmd = '%s ; %s %s' % (pre_cmds, start_up, ' '.join(args))
    else:
        term_cmd = '%s %s' % (start_up, ' '.join(args))
    os_system(term_cmd, redirect=2, background=1)


# Pop up a top level window to collect and execute user's shell command.
# Can be supplied either a single path or a list of paths.  In the list
# case, can also invoke a separate editor window to filter the list.
# Also supports writing updated list to a file.

def _create_cmd_window(fr, message, header, path_or_list,
                       invoke_cmd, write_paths_fn=null_proc):
    list_case = isinstance(path_or_list, list)
    top_fr = Frame(fr)
    Label(top_fr, text=header).pack(side=LEFT, pady=5)
    
    # edit_state survives initial invocation of this function
    # list_case: it holds [active_list, hidden_list, edit window object]
    
    edit_state = [ None, None, None ]
    if list_case:
        edit_state = [ path_or_list[:], [], None ]
        edit_cmd = EventHandler('Edit Path List for Shell Command',
                                lambda : _path_list_editor(edit_state,
                                                           write_paths_fn))
        MediumButton(top_fr, text='View/Edit Path List',
                     command=edit_cmd).pack(side=LEFT, padx=20)
    else:
        path_entry = Entry(top_fr, width=80, highlightthickness=0,
                           background=colors['text_bg_color'])
        path_entry.insert(0, path_or_list)
        path_entry.pack(side=LEFT, padx=5)
    top_fr.pack()
    
    Message(fr, text=display_messages[message], foreground='dark blue',
            aspect=1500, justify=LEFT).pack(pady=10)
    cmd_fr = Frame(fr)
    Label(cmd_fr, text='Command:').pack(side=LEFT)
    entry = Entry(cmd_fr, width=70, highlightthickness=0,
                  background=colors['text_bg_color'])
    entry.pack(side=LEFT, padx=5, fill=X, expand=YES)
    win_state = [0,0,0]    # [after_id, cmd_pid, cmd_queue]
    
    def invoke_shell(cmd):
        cmd_out.delete('1.0', END)
        fr.update()
        if list_case: pth = edit_state[0]
        else:         pth = path_entry.get()
        result = invoke_cmd(cmd, pth, timeout_entry.get())
        if isinstance(result, str):
            cmd_out.insert(END, result)
        else:
            win_state[1:3] = result      # (cmd_pid, cmd_queue)
            buttons[0]['state'] = NORMAL
            wait_for_cmd(8)

    def wait_for_cmd(delay):
        try:
            cmd_output = win_state[2].get_nowait()
        except Queue.Empty:
            if delay < 250: delay *= 2
            win_state[0] = fr.after(delay, wait_for_cmd, delay)
            return
        try:
            output = parse_cmd_out(cmd_output)
        except CommandTimeout, err_msg:
            output = '***** %s' % err_msg
        except CommandAbort, err_msg:
            output = '***** %s' % err_msg
        except CommandFailure, err_msg:
            output = '***** %s' % err_msg
        buttons[0]['state'] = DISABLED
        cmd_out.insert(END, output)
        win_state[0] = 0

    submit_cmd = EventHandler('Pipe Path(s) to Shell Command',
                              lambda *args: invoke_shell(entry.get()), fr)
    entry.bind('<Return>', submit_cmd)
    ThinButton(cmd_fr, text='Submit', min_width=1,
               command=submit_cmd).pack(side=LEFT, padx=3)
    cmd_fr.pack(pady=5, fill=X)
    out_fr, cmd_out = \
            scrolled_text_widget(fr, width=80, height=20, wrap=WORD,
                                 horiz_scroll=1, font=small_fixed_font)
    out_fr.pack(padx=2, pady=5, fill=BOTH, expand=YES)

    bottom_fr = Frame(fr)
    Label(bottom_fr, text='Timeout (seconds, 0 or blank to disable):'
          ).pack(side=LEFT)
    timeout_entry = Entry(bottom_fr, width=5, highlightthickness=0,
                          justify=RIGHT, background=colors['text_bg_color'])
    timeout_entry.insert(0, shell_cmd_timeout)
    timeout_entry.pack(side=LEFT, padx=10)
    
    def close_out():
        if edit_state[2]: edit_state[2].destroy()
        fr.destroy()
    def abort_cmd():   # can only be invoked when button state = normal
        abort_spawned_cmd(win_state[1])
        buttons[0]['state'] = DISABLED
    but_fr, buttons = \
        button_row(bottom_fr, (('Abort', abort_cmd, 0),
                               ('Close', close_out, 0),), padx=10)
    buttons[0]['activeforeground'] = 'red'
    buttons[0]['state'] = DISABLED
    but_fr.pack(side=RIGHT, padx=10, pady=2)
    bottom_fr.pack()
    entry.focus_set()

# An edit window is displayed on request for path lists.  The user can
# manipulate an active and hidden path list, which are saved in the
# parent window's state (variable path_list_list).

def _path_list_editor(path_list_list, write_paths_fn):
    def make_window(fr):
        Message(fr, text=display_messages['edit_path_list'],
                foreground='dark blue',
                aspect=1500, justify=LEFT).pack(pady=10)
        Label(fr, text='Active Path List:').pack(pady=2)
        fr_a, active = scrolled_listbox(fr, width=80, height=18,
                                        font=small_fixed_font,
                                        horiz_scroll=1, selectmode=EXTENDED)
        for p in path_list_list[0]: active.insert(END, p)
        fr_a.pack(padx=5, pady=5)
        def move_items(from_listbox, to_listbox):
            moving = list(from_listbox.curselection())
            if not moving: return
            for i in moving:
                to_listbox.insert(END, from_listbox.get(i))
            moving.reverse()
            for i in moving: from_listbox.delete(i)
            to_listbox.see(END)

        action_fr = Frame(fr)
        MediumButton(action_fr, text='Hide Selected Paths %s' % u_wtrigdn,
                     command=lambda : move_items(active, hidden)
                     ).pack(side=LEFT, padx=10)
        MediumButton(action_fr, text='Restore Selected Paths %s' % u_wtrigup,
                     command=lambda : move_items(hidden, active)
                     ).pack(side=LEFT, padx=10)
        action_fr.pack(pady=8)
        Label(fr, text='Hidden Path List:').pack(pady=2)
        fr_h, hidden = scrolled_listbox(fr, width=80, height=18,
                                        font=small_fixed_font,
                                        horiz_scroll=1, selectmode=EXTENDED)
        for p in path_list_list[1]: hidden.insert(END, p)
        fr_h.pack(padx=5, pady=5)
        
        def update_path_list():
            path_list_list[0] = active.get(0, END)
            path_list_list[1] = hidden.get(0, END)
            fr.destroy()
        def copy_path_list():
            copy_text_clipboard(' '.join(active.get(0, END)))
        button_row(fr, (('Install Active Paths', update_path_list, 0),
                        ('Save Paths As...',
                         lambda : write_paths_fn(active.get(0, END)), 0),
                        ('Copy Path List', copy_path_list, 0),
                        ('Cancel', lambda : fr.destroy(), 0),),
                   padx=5)[0].pack(pady=5)
    path_list_list[2] = \
        pop_up_toplevel_window(make_window, (),
                               'View or Edit Active Path List',
                               top_margin=5, bottom_margin=0)


# Invoke an external shell command on a path (file or directory).

def invoke_cmd_on_path(direc, orig_path):
    if direc:
        path_type = 'Directory'
        message = 'invoke_shell_command_direc'
    else:
        path_type = 'File'
        message = 'invoke_shell_command_file'
    def make_window(fr):
        def invoke_cmd(cmd, path, timeout=''):
            if direc:
                cd_cmd = 'cd "%s"; %s' % (path, cmd)
                shell_out = shell_command_with_timeout(cd_cmd, timeout)()
            elif '%s' in cmd:
                shell_out = shell_command_with_timeout(cmd, timeout)(path)
            else:
                if cmd.strip(): cmd = 'cat %s | ' + cmd
                shell_out = shell_command_with_timeout(cmd, timeout)(path)
            return shell_out
        _create_cmd_window(fr, message, 'Path: ',  ### '%s: ' % path_type,
                           orig_path, invoke_cmd)
    pop_up_toplevel_window(make_window, (),
                           'Invoke Shell Command on %s' % path_type,
                           top_margin=5, bottom_margin=0)

# Invoke an external shell command on a path list

def invoke_cmd_on_path_list(write_paths_fn, orig_path_list):
    def make_window(fr):
        def invoke_cmd(cmd, path_list, timeout=''):
            paths = ' '.join([ '"%s"' % p for p in path_list ])
            return shell_command_with_timeout(cmd, timeout)(paths)
        _create_cmd_window(fr, 'invoke_shell_command_paths',
                           'A list of %d paths is available for substitution.'
                           % len(orig_path_list), orig_path_list,
                           invoke_cmd, write_paths_fn)
    pop_up_toplevel_window(make_window, (),
                           'Invoke Shell Command on Path List',
                           top_margin=5, bottom_margin=0)

