# Includes classes and methods for displaying query results as well as
# setting up all GUI structures used to find search terms within file
# displays, select actions from context menus, do additional regular
# expression searches on text, etc.


import os, sys, string, glob
from time import strftime, localtime, asctime

from client_globals import *
from widget_utils   import *

from file_type_attribs import *
from db_operations     import *
from find_state        import many_threshold


tag_margin = 4


#-----------------------------------------

# Note: these procedures were originally methods of QP_GUI_elements.

# Format content headline items after query results are returned.
# Each item will be described by a title line, one or more word context
# lines, and a path/info line.  Tk Text widget tags are used to identify
# and color alternating regions.

def format_headlines(self, items, q_parsed, view_size):
        # Item tuples:  (file_id, abs_path, file_type, rank)
        # Tags created: headline_<item_num>, headline_text
        #               title_<item_num>, path_<item_num>
        #               term_<item_num>_<term_num>
        win = self.path_widget
        win.configure(state=NORMAL)
        view_start = self.num_qlines
        if view_size == -1: view_size = len(items) - view_start  # do the rest
        if 0 < view_start < len(items):
            see_pt = win.index(END)
        else:
            see_pt = None
        if view_start == 0:
            self.headlines = []
            self.item_button_tags = []
            self.content_terms = q_parsed
        headline_bg = (colors['headline_alt'], 'white')

        # When an item is selected (clicked in its region), the file is
	# displayed and the item is highlighted.
	
        def item_action(item_num, term_num, title_text):
            win.configure(state=NORMAL)
            if self.selected_index != None:
                win.tag_configure('headline_%s' % self.selected_index,
                        background=headline_bg[self.selected_index % 2])
            fetch = item_num != self.selected_index
            win.tag_configure('headline_%s' % item_num,
                              background=colors['headline_select'])
            self.view_file(1, 1, fetch, item_num, 0, title_text)
            self.selected_index = item_num    # set after calling view_file
            win.configure(state=DISABLED)
            eval_assertion(self.state_invariant, 'Post-action check')

        # If the title line is clicked, the occurrence counts are cleared
	# in addition to normal item actions.
	
        def title_action(item_num, term_num, title_text, mod_time):
            # reset occurrence and highlighting
            win.configure(state=NORMAL)
            if self.find_state[item_num]:
                self.find_state[item_num].clear_occurrence()
            self.find_state[item_num] = None
            self.after_display_action = lambda : self.display.see(1.0)
            for tag in self.item_button_tags[item_num]:
                win.tag_configure(tag, foreground='red')
            win.configure(state=DISABLED)

        lex_terms = [ (q, lexize_term(q)) for q in q_parsed
                      if q[0] not in '!-' ]

	# The text for each search term is turned into a "button" by
	# creating a tag and binding actions to it.
	
        def add_term_button(term_num, item_num, term_start, term_end,
                            button_action):
            term_tag = 'term_%s_%s' % (item_num, term_num)
            win.tag_add(term_tag, term_start, term_end)
            win.tag_configure(term_tag, foreground='red')
            self.item_button_tags[item_num].append(term_tag)
            def invoke_button(delta=1):
                win.configure(state=NORMAL)
                for tag in self.item_button_tags[item_num]:
                    win.tag_configure(tag, foreground='red')
                # schedule advance_to_term call to happen after view_file
                self.after_display_action = \
                    lambda args=(item_num, term_num, delta): \
                        advance_to_term(self, *args)
                win.tag_configure(term_tag,
                                  foreground=colors['find_text_color'])
                win.configure(state=DISABLED)
            if button_action:
                win.tag_bind(term_tag, '<Button-1>',
                             EventHandler('Headline button forward',
                                          lambda dummy: invoke_button(1)))
                win.tag_bind(term_tag, '<Shift-Button-1>',
                             EventHandler('Headline button backward',
                                          lambda dummy: invoke_button(-1)))

	# All search terms, which have been demarcated using <b>...</b>,
	# are highlighted in read.
	
        def highlight_terms(item_start, item_end, item_num, actuals,
                            button_action=0, check_missing=1):
            term_start = win.search('<b>', item_start, item_end)
            while term_start:
                win.delete(term_start, '%s + 3 chars' % term_start)
                term_end = win.search('</b>', term_start, item_end)
                win.delete(term_end, '%s + 4 chars' % term_end)
                term_str = win.get(term_start, term_end)
                if term_str not in actuals:      # omit duplicates
                    actuals.append(term_str)
                add_term_button(actuals.index(term_str), item_num,
                                term_start, term_end, button_action)
                term_start = win.search('<b>', term_end, item_end)
            if not check_missing: return
            # check for missing terms and add to end
            aterms = [ lexize_term(q) for q in actuals if q[0] not in '!-' ]
            missing = [ t[0] for t in lex_terms if t[1] not in aterms ]
            return missing

        # Headlines are computed in the server and processed here.
	
        def insert_headline(head_start, item_num, path, type_class,
                            specials, actuals, tags, button_action):
            all_tags = tags + ('headline_text',)
	    first_part = get_headline(type_class, path,
				      self.content_terms, 20, specials)
            win.insert(head_start, '    ' + first_part, all_tags)
            head_mid = win.index('%s lineend' % head_start)
            missing_terms = \
                highlight_terms(head_start, head_mid,
                                item_num, actuals, button_action)
            if not missing_terms: return
            remaining = \
                get_headline(type_class, path, missing_terms, 10, specials, 1)
            if remaining:
	        if first_part: spacer = '  ...  '
		else:          spacer = ''
	        win.insert(head_mid, spacer + remaining, all_tags)
            highlight_terms(head_mid, '%s lineend' % head_mid,
                            item_num, actuals, button_action, 0)
	    ### ignore for now any residual missing terms

	# For each item we look up its file attributes and add a line
	# that summarizes them along with the search rank value.
	
        initial_action = null_proc
        item_num = view_start
        item_start = win.index('end - 1 char')
        for item in items[view_start:view_start+view_size]:
            title_tag = 'title_%s' % item_num
            path_tag = 'path_%s' % item_num
            item_tag = 'headline_%s' % item_num
            abs_path, file_type, rank = item[1:]
            type_class = file_type_class[file_type]
            specials = type_class.identifier_specials
            if os.path.exists(abs_path):
                file_len = os.path.getsize(abs_path)
                mod_time = os.path.getmtime(abs_path)
                fdate = strftime('%b %d %Y', localtime(mod_time))
                extension = os.path.splitext(abs_path)[1][1:]
                ### handle extensions?
                # need to check for blank title because method might
		# return empty string
		title_text = type_class.extract_title(abs_path) or \
			     os.path.basename(abs_path)
            else:
                file_len, fdate = -1, 'Deleted    '
                title_text, mod_time = abs_path, 0

            win.insert(END, '%d. %s\n' % (item_num+1, title_text), title_tag)
            win.tag_configure(title_tag, foreground=colors['item_meta_fg'],
                              font=title_font, spacing1=2)
            self.item_button_tags.append([])
            term_actuals = []
            if file_len > 0:
                insert_headline(END, item_num, abs_path, type_class,
                                specials, term_actuals, (), 1)
            win.insert(END, '\n')
            win.insert(END, '%6.2f  %s  %s  %s  %s' %
		            (rank, file_type, approx_size(file_len),
			     fdate, abs_path),
                       path_tag)
            win.tag_configure(path_tag, foreground=colors['muted_red'],
                              font=headline_font, spacing3=4)
                    
            win.insert(END, '\n')
#            win.insert(END, '\n', 'separators')
            win.tag_add(item_tag, item_start, win.index('end - 1 char'))
            if item_num % 2: tag_bg = 'white'
            else:            tag_bg = colors['headline_alt']
            win.tag_configure(item_tag, background=tag_bg, rmargin=tag_margin,
                              lmargin1=tag_margin, lmargin2=tag_margin)
            win.tag_bind(item_tag, '<Button-1>',
                         EventHandler('Content file select',
                                      lambda dummy, args=(item_num, 0,
                                                          title_text):
                                          item_action(*args)))
            win.tag_bind(title_tag, '<Button-1>',
                         EventHandler('Content file title select',
                             lambda dummy, args=(item_num, 0,
                                                 title_text, mod_time):
                                      title_action(*args)))
            headline_context_menu(self, win.tag_bind, item_tag, item_num)
            
            self.headlines.append((abs_path, file_len, term_actuals))
            if item_num == view_start:    # display highest ranked item
                initial_action = \
                    lambda args=(item_num, 0, title_text): item_action(*args)
            item_num += 1
            item_start = win.index('end - 1 char')

#        win.tag_configure('separators', font=spacer_font)
        win.tag_configure('headline_text', font=headline_font)
        if see_pt: win.see(see_pt)
        initial_action()
        win.configure(state=DISABLED)
        self.num_qlines = min(view_start + view_size, len(items))
        self.qlines_count.config(text='%d' % self.num_qlines)
        self.adjust_continue_buttons()

# Each headline item gets a right button binding for a popup menu.

def headline_context_menu(self, binding_function, tag, index):
        def pop_up(event):
            self.popup_index = index
            self.popup_handler(event)
        binding_function(tag, '<Button-3>',
                         EventHandler('Content popup context menu', pop_up, 0))


# Following is used to implement the "button" search feature of headlines.
# Clicking on a search term in a headline causes the next occurrence to
# be found in the display window.  Shares features with the auxiliary
# search function (Find entry).

def advance_to_term(self, item_num, term_num, delta):
        # assumes term sites and cur_site already set for this file
        # creates tag highlight_line in self.display
        find_state = self.find_state[item_num]
        instance_count = self.cur_site[term_num] + delta + 1
        total_count = self.hterm_total_count[term_num]
        if total_count == 0:
            find_state.highlight_line(0, 'missing_%s' % term_num)
            return
        if not (1 <= instance_count <= total_count):
            instance_count -= total_count * delta    # delta = +/- 1
        site = self.sites[term_num][instance_count - 1]
        self.cur_site[term_num] = instance_count - 1
        if total_count >= many_threshold:
            total_count = 'many'
        find_state.show_occurrence(0, delta, instance_count, total_count)
        new_tag = 'actual_%s_%s' % (term_num, instance_count - 1)
        find_state.highlight_line(site, new_tag)
        find_state.prev_index = 0
