
# Menubar actions from the main application window are handled by
# functions in this module.


import os, sys, time, string, glob, re

from client_globals import *
from widget_utils   import *
from db_operations  import *

from file_type_attribs import *

from indexing_manager import scan_for_files


# ( Following procedures previously had been methods of class GUI_elements. )


excluded_profiles = ('null_profile', 'profile_template',
                     'template_howto', 'test')

helpDict = None     # help topics not loaded until needed


# Users may change preferences by editing the preference file within a
# text widget. New preferences take effect during the next session.

def change_preferences():
        def add_buttons(fr):
            button_row(fr, (('Save Preferences', lambda : save_prefs(), fr),
                            ('Restore Defaults',
                             lambda : restore_defaults(), fr),
                            ('Cancel', lambda : top.destroy())),
                       'Prefs:')[0].pack(pady=10)
        top, txt = scrolled_text_display(
                       title='Edit Preferences for Pathena Client',
                       button_proc=add_buttons, # tab_stops=('120','180'),
                       intro=display_messages['edit_prefs'], aspect=800,
                       width=80, height=40,
                       font=small_fixed_font, return_frame=1)
        def load_prefs(dir_name, file_name):
            try:
                pref_file = open(os.path.join(dir_name, file_name))
                lines = pref_file.readlines()
                pref_file.close()
            except IOError:
                lines = []           # OK for file to be missing
            for line in lines: txt.insert(END, line)
            txt.yview_moveto(0.0)
        def save_prefs():
            pref_file = open(os.path.join(config_dir,
                                          'preferences.conf'), 'w')
            pref_file.writelines(txt.get(1.0, txt.index(END) + '- 1 chars'))
            pref_file.close()
            emit_log_entry(main_log_file, 'Preferences file updated.')
            top.destroy()
        def restore_defaults():
            txt.delete(1.0, END)
            load_prefs(config_dir, 'orig_prefs.conf')
        load_prefs(config_dir, 'preferences.conf')

# Users may customize source file types by editing the class file within a
# text widget. New file types take effect during the next session.

def customize_source_types():
        def add_buttons(fr):
            button_row(fr, (('Save Source Types', lambda : save_custom(), fr),
                            ('Cancel', lambda : top.destroy())),
                       'Custom_source:')[0].pack(pady=10)
        top, txt = scrolled_text_display(
                       title='Customize Source File Types for Pathena',
                       button_proc=add_buttons, # tab_stops=('120','180'),
                       intro=display_messages['customize_source'],
                       width=80, height=40, aspect=900,
                       font=small_fixed_font, return_frame=1)
        def load_file(dir_name, file_name):
            try:
                edit_file = open(os.path.join(dir_name, file_name))
                lines = edit_file.readlines()
                edit_file.close()
            except IOError:
                lines = []           # OK for file to be missing
            for line in lines: txt.insert(END, line)
            txt.yview_moveto(0.0)
        def save_custom():
            edit_file = open(os.path.join(config_dir, 'source_custom.py'), 'w')
            edit_file.writelines(txt.get(1.0, txt.index(END) + '- 1 chars'))
            edit_file.close()
            emit_log_entry(main_log_file, 'Custom source types updated.')
            top.destroy()
        load_file(config_dir, 'source_custom.py')


# Indexing profiles may be established or edited by the user through
# this dialog and edit sequence.

def change_profiles():
        profile_path = askopenfilename(title='Select Indexing Profile',
                                       initialdir=profile_dir)
        if not profile_path: return
        profile_name = os.path.basename(profile_path)
        def add_buttons(fr):
            button_row(fr, (('Save Profile',
                             lambda : save_profile(profile_path), fr),
                            ('Save Profile As...',
                             lambda : save_profile_as(), fr),
                            ('Cancel', lambda : top.destroy())),
                       'Profile:')[0].pack(pady=10)
        top, txt = scrolled_text_display(
                       title="Edit Indexing Profile '%s'" % profile_name,
                       button_proc=add_buttons, # tab_stops=('120','180'),
                       intro=display_messages['edit_profile'], aspect=900,
                       width=80, height=50,
                       font=small_fixed_font, return_frame=1)
        def load_profile(path_name):
            try:
                profile_file = open(path_name)
                lines = profile_file.readlines()
                profile_file.close()
            except IOError:
                lines = []           # OK for file to be missing
            for line in lines: txt.insert(END, line)
            txt.yview_moveto(0.0)
        def save_profile(path):
            profile_file = open(path, 'w')
            profile_file.writelines(txt.get(1.0, txt.index(END) + '- 1 chars'))
            profile_file.close()
            emit_log_entry(main_log_file,
                           "Edited indexing profile '%s'." % path)
            top.destroy()
        def save_profile_as():
            new_path = asksaveasfilename(title='Select New Profile Name',
                                         initialdir=profile_dir)
            if new_path: save_profile(new_path)
        load_profile(profile_path)


# Users may check the status of the database server.  A popup display
# showing a few statistics is provided in response.

def checkServerStatus():
        def make_window(fr):
            Label(fr, text='Pathena Database Server',
                  font=heading_font).pack(pady=10)
            status = getServerStatus()
            if status:
                status_msg = 'Pathena Server is up and running on %s\n' \
                    'under the control of a PostgreSQL postmaster process.'
            else:
                status_msg = 'Pathena Server is not running on %s.'
            Label(fr, font=text_font, justify=LEFT,
                  text= status_msg % time.asctime()).pack(padx=5, pady=5)
            if status:
                fr2, cum_stats = \
                     build_table(fr, (lambda w,t: Label(w, text=t),
                                      lambda w,n: Label(w, text=str(n))),
                                 zip(('Postmaster process id:',
                                      'Listening on port:',
                                      'Approximate database size (MB):   ',
                                      'PostgreSQL cluster size (MB):',
                                      'Paths indexed in the database:',
                                      'Files having full text indexing:',
                                      'Files cached in the database:',
                                      ), status),
                                 (W, E))
                fr2.pack(pady=10)
                emit_log_entry(main_log_file, 'Retrieved database status.')
            MediumButton(fr, text='Close',
                         command=EventHandler('Status Close',
                                              lambda : fr.destroy(), 0)
                         ).pack(pady=5)
        pop_up_toplevel_window(make_window, (), 'Pathena Server Status',
                               bottom_margin=5)

# Compute and display statistics from Tsearch2 after collecting a few
# options from user.  The number of histogram entries may be specified
# as well as which end of the distribution to sample from.  When the
# number of entries is high, the computation time is significant.

def showTsearchStats(stat_type):
        title = 'Indexed Word Statistics (%s)' %  stat_type.capitalize()
        def make_window(fr, vector):
            Message(fr, aspect=500, text=display_messages['display_stats'],
                    font=text_font).pack(padx=10)
            show_fr = Frame(fr)
            Label(show_fr, text='No. of words: ').pack(side=LEFT)
            word_count = Entry(show_fr, width=7, justify=RIGHT,
                               highlightthickness=0,
                               background=colors['text_bg_color'])
            word_count.insert(END, '100')
            word_count.pack(side=LEFT, padx=5)
            button_row(show_fr,
                       (('Most Frequent',  lambda : show_stats(1), fr),
                        ('Least Frequent', lambda : show_stats(0), fr))
                       )[0].pack(side=LEFT, padx=10)
            show_fr.pack(pady=10)

            fr2 = Frame(fr)
            header = Frame(fr2)
            Label(header, text='                 Word').pack(side=LEFT)
            Label(header, text='No. entries      ').pack(side=RIGHT)
            Label(header, text='No. files        ').pack(side=RIGHT)
            header.pack(fill=X, pady=5)
            stops = tuple([ t * fixed_text_pixels for t in (30, 40) ])
            table_fr, table = \
                scrolled_text_widget(fr2, width=50, tab_stops=stops,
                                     font=fixed_text_font)
            table_fr.pack(padx=10, pady=5, fill=X)
            fr2.pack(pady=5, fill=X)
            fr3, cum_stats = \
                 build_table(fr2, (lambda w,t: Label(w, text=t),) * 2,
                             (('Total number of words', '0'),
                              ('Number of singleton words', '0'),
                              ('Average number of files per word','0'),
                              ('Average number of entries per word   ', '0')),
                             (W, E))
            fr3.pack(pady=5)
	    
            def show_stats(freq=1):
                table.delete('1.0', END)
                fr.update_idletasks()    # allow for delay in collecting stats
                stats = getTsearchStats(vector, int(word_count.get()), freq)
                if freq: word_num, incr = 1, 1
                else:    word_num, incr = stats[0][0], -1
                for word, ndocs, nentries in stats[1]:
                    word = restore_unencoded_text(word)
                    table.insert(END, '%6d. %s\t' % (word_num, word))
                    table.insert(END, '%8d\t' % ndocs, 'num_col')
                    table.insert(END, '%8d\n' % nentries, 'num_col')
                    word_num += incr
                for row, fmt, val in \
                        zip(cum_stats,
                            ('%d', '%d', '%7.2f', '%7.2f'), stats[0]):
                    row[1].configure(text=(fmt % val))
            Message(fr, aspect=2000, text=display_messages['about_stat_update']
                    ).pack(padx=10, pady=5)
            button_row(fr, (('Close', lambda :fr.destroy(), 0),)
                       )[0].pack(pady=5)
            show_stats()
            emit_log_entry(main_log_file, 'Show %s word statistics.' % vector)
        pop_up_toplevel_window(make_window, (stat_type,), title,
                               bottom_margin=5)

# Background indexing is initiated here but the main work is performed
# by module indexing_manager.

def index_directories():
        def make_window(fr):
            Message(fr, aspect=250, text=display_messages['index_files'],
                    font=text_font).pack(padx=10, pady=10)
            button_row(fr, (('Full Indexing',
                             lambda : scan_for_files(fr), fr),
                            ('Quick Indexing',
                             lambda : scan_for_files(fr, 0, 0), fr),
                            ('Cancel', lambda : fr.destroy(), 0))
                       )[0].pack(pady=5)
	# window is moved up to prevent file dialog from being obscured by it
        pop_up_toplevel_window(make_window, (), 'Pathena File Indexing',
			       y_offset=-100)
    
# Clearing (erasing) the database is handled by a separate script.

def clear_database():
        def make_window(fr):
            Message(fr, aspect=500, text=display_messages['clear_database'],
                    font=text_font).pack(padx=10)
            def clear_db():
                progress = Label(fr, text='Fully clearing database...')
                progress.pack(pady=10)
                fr.update_idletasks()
                os_system(os.path.join(bin_dir, 'clear_database'))
                progress.destroy()
                user_dialog(showinfo, 'Clear database',
                            'Database has been cleared.')
                emit_log_entry(main_log_file, 'Clear full database.')
                fr.destroy()
            button_row(fr, (('Clear Indexed File Database', clear_db, fr),
                            ('Cancel', lambda : fr.destroy(), 0))
                       )[0].pack(pady=15)
        pop_up_toplevel_window(make_window, (), 'Clear Pathena Database',
                               bottom_margin=0)

# A dialog helps the user construct entries for crontab editing.  Sample
# entries are copied to the clipboard, after which they can be inserted
# into the user's cron table.

def schedule_indexing():
        def make_window(fr):
            Message(fr, aspect=200, text=display_messages['schedule_indexing'],
                    font=text_font).pack(padx=10, pady=10)
            cfr, crontab = scrolled_text_widget(fr, width=60, height=4,
                                                horiz_scroll=1)
            all_profiles = os.listdir(os.path.join(os.environ['PATHENA'],
                                                   'profiles'))
            profiles = [ p for p in all_profiles
                         if not (p in excluded_profiles or p.endswith('~')) ]
            crontab.insert(END, '# Pathena periodic batch indexing:\n'
                                '# End of Pathena indexing entries\n')
            def insert_entry(suffix):
                crontab.insert('end - 2 lines',
                               '15 3 * * * %s/bin/batch_index_%s %s\n' %
                               (os.environ['PATHENA'], suffix,
                                ' '.join(profiles)))
            cfr.pack(padx=10, pady=10)
            button_row(fr, (('Insert Full Indexing Entry',
                             lambda : insert_entry('full'), 0),
                            ('Insert Quick Indexing Entry',
                             lambda : insert_entry('quick'), 0),)
                       )[0].pack(pady=5)
            button_row(fr, (('Copy Text of Crontab Entries',
                             lambda : copy_text_clipboard(
                                          crontab.get('1.0', END)), 0),
                            ('Close', lambda : fr.destroy(), 0),)
                       )[0].pack(pady=5)
        pop_up_toplevel_window(make_window, (), 'Scheduling Pathena Indexing',
                               bottom_margin=5)



# Indexable files are defined by modules in directory ~/.pathena/types.
# This display shows genus, species and extensions for the included
# file types.  It also checks whether all the required OS commands and
# other utilities are available, then reports on any that are missing.

def show_file_types():
        title = 'Indexable File Types'
        file_types = [ (c.genus_name, c.species_name, c.file_extensions)
                       for c in file_type_class.values() if c.species_name ]
        file_types.sort()
        file_cmds =  [ (c.genus_name, c.species_name, c.commands_needed)
                       for c in file_type_class.values() if c.commands_needed ]
        file_cmds.sort()
        def make_window(fr):
            Message(fr, aspect=750, font=text_font,
                    text=display_messages['display_file_types']).pack(padx=10)
            stops = tuple([ t * fixed_text_pixels for t in (12, 22) ])
            fr2, table_fr2, type_table = \
                 scrolled_table_text(fr, width=60, height=20, tab_stops=stops,
                     header_labels=('           Genus',
                                    '        Species',
                                    '          File Extensions'))
            fr2.pack(padx=10, pady=5)
            item_num = 1
            for genus, species, extensions in file_types:
                type_table.insert(END, '%3d. %s\t%s\t%s\n' %
                                  (item_num, genus, species, list(extensions)))
                item_num += 1
            num_exts = reduce(add, [ len(t[2]) for t in file_types], 0)
            fr3, cum_stats = \
                 build_table(fr2, (lambda w,t: Label(w, text=t),) * 2,
                             (('Total number of file types',
                               str(len(file_types))),
                              ('Total number of file extensions      ',
                               str(num_exts))), (W, E))
            fr3.pack(pady=5)

            missing_cmds = []
            for genus, species, cmds in file_cmds:
                missing = [ c for c in cmds if check_os_command(c) ]
                if missing:
                    missing_cmds.append((1 + len(missing_cmds),
                                         genus, species, missing))
            if missing_cmds:
                if backup_conversion:
                    missing_msg = display_messages['missing_cmds_backup']
                else:
                    missing_msg = display_messages['missing_file_cmds']
                Message(fr, aspect=650, font=text_font,
			text=missing_msg).pack(padx=10)
                fr4, table_fr4, command_table = \
                     scrolled_table_text(fr, width=60, tab_stops=stops,
                         height=min(8, max(4, len(missing_cmds))),
                         header_labels=('           Genus',
                                        '        Species',
                                        '          Missing Commands'))
                fr4.pack(padx=10, pady=5)
                for mc in missing_cmds:
                    command_table.insert(END, '%3d. %s\t%s\t%s\n' % mc)
            else:
                Message(fr, aspect=1000, font=text_font,
                        text=display_messages['display_file_cmds_present']
                        ).pack(padx=10, pady=5)

            button_row(fr, (('Close', lambda :fr.destroy(), 0),)
                       )[0].pack(pady=10)
        pop_up_toplevel_window(make_window, (), title, bottom_margin=0)


# Help menu features are implemented below.

def about_window():
    def make_window(fr):
        Label(fr, text='Pathena Desktop Search',
              font=heading_font).pack(padx=50, pady=10)
        Message(fr, aspect=1000, justify=CENTER,
                text=about_text).pack(padx=20)
        MediumButton(fr, text='Close',
                     command=EventHandler('About Close',
                                          lambda : fr.destroy(), 0)
                     ).pack(pady=5)
    pop_up_toplevel_window(make_window, (), 'About Pathena Client',
                           bottom_margin=5)

def quick_start():
    import quick_start
    def add_buttons(fr):
        button_row(fr, (('Close', lambda : fr.destroy(), 0),))[0].pack()
    txt = scrolled_text_display(title='Quick Start Guide for Pathena Client',
                                font=help_text_font,
                                tab_stops=('.5c','1c','10c'),
                                button_proc=add_buttons)
    txt.insert(END, quick_start.quick_text % app_version)


# The user's guide is loaded only on demand when the user requests
# display of the help window.  After loading, it will be saved for
# subsequent viewing.

def help_desk():
    if not helpDict: load_help()
    fr = Toplevel()
    fr.geometry('+0+0')
    fr.title(string="Pathena User's Guide")
    Frame(fr).pack(pady=3)
    MediumButton(fr, text='Close',
                 command=EventHandler('Close Help', lambda : fr.destroy(), 0)
                 ).pack(pady=3, side=BOTTOM)

    topics = Listbox(fr, selectmode=SINGLE, font=text_font,
		     exportselection=0, background=colors['text_bg_color'],
		     height=30, width=20, highlightthickness=0)
    topics.pack(side=LEFT, padx=3, pady=3, expand=YES, fill=Y)
    
    for item in helpTopics: topics.insert(END, item)
    txt_fr, txt = scrolled_text_widget(fr, width=60, height=30,
                                       font=help_text_font)
    txt_fr.pack(side=LEFT, padx=3, pady=3, expand=YES, fill=BOTH)
    selection = lambda event: help_func(event.widget, event.y, txt)
    topics.bind('<Button-1>',
                EventHandler('Display help topic', selection ,fr))
    help_func(topics, 0, txt)     # display intro

def help_func(topics, y, txt):
    name = topics.get(topics.nearest(y))
    txt.delete(1.0, END)
    item = helpDict[name]
    txt.insert(END, '%s\n\n' % name)
    txt.tag_add('topic', '1.0', '2.0')
    txt.tag_configure('topic', foreground='dark blue', justify=CENTER,
                      font=heading_font)
    if isinstance(item, str):
        txt.insert(END, item)
    else:
        txt.image_create(END, image=item)

def load_help():
    global helpDict, helpTopics
    import user_help
    helpDict = {}
    for topic, text in user_help.helpItems:
        helpDict[topic] = text
    helpTopics = [ item[0] for item in user_help.helpItems ]


about_text = '''
Version %s

Released %s

Contact: %s

http://%s
''' % (app_version, release_date, email_addr, home_page)

