
# Construction of Pathena Client's main application window begins in this
# module.  Included is the menubar layout.  The handler functions for the
# menu items are found in module menu_actions.  Query processing takes
# place in modules submit_query and query_results.


import os, sys, time, string, glob, re

from app_globals    import *
from client_globals import *
from widget_utils   import *

debug_dir = os.environ.get('PATHENA_DEBUG')
if debug_dir:
    sys.path.insert(0, debug_dir)
    from devel_utils import *

from tabbed_notebook import add_notebook_panel

from file_type_attribs import *

#root.option_add('*borderwidth', 1)

# File type handlers are kept in directory ~/.pathena/types.  Whatever
# Python files are found there will be loaded.

try:
    for handler in glob.glob(os.path.expandvars('$PATHENA/types/*.py')):
        __import__(os.path.splitext(os.path.basename(handler))[0])
except:
    emit_log_entry(main_log_file,
                   "Failed to import types module '%s'." % handler)
    _trace = StringIO()
    print_exc(1, _trace)
    emit_log_entry(main_log_file, _trace.getvalue())

# Importing db_operations causes a connection to the PostgreSQL
# database server to be established.

try:
    from db_operations import *
except ConnectionFailure, err_msg:
    user_dialog(showerror, *error_messages['db_server_down'])
    emit_log_entry(main_log_file, 'Database connection failure:\n%s' % err_msg)
    raise StartUpError

from menu_actions  import *
from submit_query  import perform_query
from query_results import Query_Page

query_entry_width = 36

# Initialize server-side modules for handling client-initiated queries.

do_query_effect("select init_client_ops()")


#-----------------------------------------

# Create main window for launching queries.  Window built on start-up
# and remains up throughout Pathena session (unless user kills it).

class GUI_elements(object):       # GUI objects, including those accessed
                                  # during session; initialize only once
    def __init__(self, master, app, menubar_items):
        self.master = master
        master.title('Pathena Desktop Search')
        menubar_row = Frame(master)
        self.add_menubar(menubar_row, app, menubar_items)
        menubar_row.pack(padx=0, fill=X)
        self.create_query_control_area(master)
        self.query_history = []   # collect previous queries for listbox
        master.geometry(newGeometry='-0+0')
        master.deiconify()

    # Menubars are specified using a list/tuple data structure of the form:
    #   ((<menubar item name>,
    #     (<submenu item name>, <handler function>, [<control key>]), ...),
    #    ...)   # a submenu item of None yields a separator
    
    def add_menubar(self, fr, app, menubar_items):   #
        self.menubar = Frame(fr)
        def make_menu(parent, choices):
            menu = Menu(parent, tearoff=0)#, font=menu_font)
            for ch in choices:
                if ch == None:
                    menu.add_separator()
                elif len(ch) == 1:  # disabled menu item
                    menu.add_command(label=ch[0], state=DISABLED)
                elif callable(ch[1]):      # normal menu item
                   # ch = (label, cmd, optional ctrl char, optional type, on?)
                   # type in 0 (checkbutton), 1 (radiobutton)
                    label = ch[0]+'    '
                    wrapped_cmd = EventHandler('Menubar %s' % ch[0], ch[1])
                    if len(ch) == 2:
                        menu.add_command(label=label, command=wrapped_cmd)
                    elif len(ch) >= 4 and ch[3]==0:
		        # Later, add support for keyboard shortcuts
                        menu.add_checkbutton(label=label, command=wrapped_cmd)
                        if len(ch) >= 5 and ch[4]: menu.invoke(label)
                    elif len(ch) >= 4 and ch[3]==1:
                        menu.add_radiobutton(label=label, command=wrapped_cmd)
                        if len(ch) >= 5 and ch[4]: menu.invoke(label)
                    else:
                        app.master.bind('<Control-%s>'%ch[2],
                                        lambda event,cmd=wrapped_cmd: cmd())
                        accel = 'Ctrl-%s' % string.upper(ch[2])
                        menu.add_command(label=label, command=wrapped_cmd,
                                         accelerator=accel)
                else:         # cascading submenu
                    menu.add_cascade(label=ch[0],
                                     menu=make_menu(menu, ch[1]))
            return menu
        def make_drop_menu(parent, name, *choices):
            button = Menubutton(parent, text=name)#, font=menubar_font)
            button.pack(side=LEFT, padx=2)
            button.configure(menu=make_menu(button, choices))
        for items in menubar_items:
            make_drop_menu(self.menubar, *items)
        self.menubar.pack(side=LEFT)  # , padx=5)


# At the top of the main window is where queries are entered and launched.
# This region contains a few Entry widgets for entering search terms
# plus the buttons needed for control.

    def create_query_control_area(self, win):
        def clear_entries():
            for ent in self.q_entries: ent.delete(0, END)
        def invoke_query(dummy=None, terms=None):
            if terms == None:
                first = self.q_entries[0].get()
                if ';' in first:
                    packed = split_and_pad_terms(first)
                    for ent, term in zip(self.q_entries, packed):
                        ent.delete(0, END)
                        ent.insert(END, term)
                terms = [ ent.get().strip() for ent in self.q_entries ]
            else:
                for ent, term in zip(self.q_entries, terms):
                    ent.delete(0, END)
                    ent.insert(END, term)
            submit_query(terms)
        self.invoke_query_handler = \
            EventHandler('Invoke query choice', invoke_query)
        button_width = 5

        def make_bottom_right(parent):
            bottom_right = Frame(parent)
            self.type_ent = Entry(bottom_right, width=20,
                                  highlightthickness=0, bg=text_bg)
            self.type_ent.pack(side=LEFT)
            self.types_pulldown_button = \
                Button(bottom_right, text=u_triagdn, padx=2, pady=0, anchor=S,
                       command=EventHandler('Display file type list',
                                            self.types_pull_down))
            self.types_pulldown_button.pack(side=LEFT, padx=2)
            self.submit_function_button = \
                ThinButton(bottom_right, text='Search', width=button_width,
                           command=self.invoke_query_handler)
            self.submit_function_button.pack(side=RIGHT, pady=2)
            return bottom_right

        def col_4(parent, row):
            if row == 0:
                return Entry(parent, highlightthickness=0,
                             width=query_entry_width, bg=text_bg)
            else:
                return make_bottom_right(parent)
        def col_6(parent, row):
            if row == 0:
                return ThinButton(parent, text='Clear', width=button_width,
                                  command=EventHandler('Clear entries',
                                                       clear_entries, 0))
            else:
                return ThinButton(parent, text=u'Redo%s' % u_triagdn,
                                  width=button_width,
                                  command=EventHandler('Display history list',
                                                       self.redo_pull_down))
        text_bg = colors['text_bg_color']
        q_frame, q_widgets = build_table(win,
            (lambda w,t: Label(w, text=t),
             lambda w,wid: Entry(w, highlightthickness=0,
                                 width=wid, bg=text_bg),
             lambda w,t: Label(w, text=t), col_4,
             lambda w,t: Frame(w), col_6),
            (('Text:', query_entry_width, ' Path:', 0, '', 0),
             ('Date:', query_entry_width, ' Type:', 1, '', 1)),
            anchors=(E, EW, E, EW, E, EW), col_weights=(0, 1, 0, 1, 0, 0),
            padx=1, pady=2)
        q_frame.pack(fill=X)
        Frame(win).pack(pady=1)
        self.q_entries = [q_widgets[0][1], q_widgets[0][3],
                          q_widgets[1][1], self.type_ent]
        for ent in self.q_entries:
            ent.bind('<Return>', self.invoke_query_handler)
        self.q_entries[0].focus_set()
        self.redo_pulldown_button = q_widgets[1][5]

        # The rest of the window is managed as a tabbed notebook, where
        # query results will be displayed.
        add_notebook_panel(win)


# The Redo button opens a pull-down menu showing a listbox of all queries
# initiated during this session.  Items are shown in reverse chronological
# order.  Selecting an item reruns its query.

    def redo_pull_down(self):
       button = self.redo_pulldown_button
       x = button.winfo_rootx() + button.winfo_width() \
           - self.master.winfo_rootx()  ###
       y = button.winfo_rooty() + button.winfo_height() \
           - self.master.winfo_rooty() + 5  ###
       fr = Frame(self.master, relief=SOLID, bd=2)
       box_height = max(8, min(20, len(self.query_history)))
       qlist = Listbox(fr, bd=0, highlightthickness=0, relief=FLAT,
                       background=colors['text_bg_color'], exportselection=0, 
                       height=box_height, width=1+query_entry_width)
       scrollbar = Scrollbar(fr, command=qlist.yview,
                             width=10, elementborderwidth=2,
                             highlightthickness=0, bd=1) # , relief=SUNKEN)
       qlist.configure(yscrollcommand=scrollbar.set)
       qlist.pack(side=LEFT, fill=BOTH, expand=YES, padx=2, pady=2)
       scrollbar.pack(side=LEFT, fill=Y, expand = YES, pady=2)
       fmt = '%s : ' + ' ;'.join((' %s',) * len(self.q_entries))
       # fmt = '%s :  %s ; %s ; %s ; %s'
       for query in self.query_history:
          qlist.insert(END, fmt % query)
          
       def recall_query(y):
          index = int(qlist.nearest(y))
          fr.destroy()
          if index < 0: return
          items = self.query_history[index]   # (num, terms_1, terms_2, ...)
          self.master.update_idletasks()     # remove pulldown listbox first
          self.invoke_query_handler(terms=items[1:])
       qlist.bind('<Button-1>',
                  EventHandler('Recall previous query',
                               lambda event: recall_query(event.y)))
       def history_motion(event):
          qlist.selection_clear(0, END)
          qlist.selection_set(int(qlist.nearest(event.y)))
       qlist.bind('<Motion>',
                  EventHandler('History list motion', history_motion, 0))
       fr.place(x=x, y=y, anchor=NE)
       def dismiss_if_out(event):
          win = event.widget
          if not str(win).startswith(str(fr)): fr.destroy()
       self.master.bind('<Button-1>', lambda event: dismiss_if_out(event))
#                        EventHandler('Dismiss pulldown list',
#                                     lambda event: dismiss_if_out(event), 0))
       ### will this binding get purged when listbox destroyed???


# Another pull-down menu is provided for the list of available file types.
# This is just a way to implement a simple combo-box, which is a widget
# type not included in Tk.

    def types_pull_down(self):
       button = self.types_pulldown_button
       x = button.winfo_rootx() - self.master.winfo_rootx() + 20  ###
       y = button.winfo_rooty() - self.master.winfo_rooty() + 25  ###
       fr = Frame(self.master, relief=SOLID, bd=2)
       tlist = Listbox(fr, bd=0, highlightthickness=0, relief=FLAT,
                       background=colors['text_bg_color'], exportselection=0, 
                       height=20, width=16)
       scrollbar = Scrollbar(fr, command=tlist.yview,
                             width=10, elementborderwidth=2,
                             highlightthickness=0, bd=1) # , relief=SUNKEN)
       tlist.configure(yscrollcommand=scrollbar.set)
       tlist.pack(side=LEFT, fill=BOTH, expand=YES, padx=2, pady=2)
       scrollbar.pack(side=LEFT, fill=Y, expand = YES, pady=2)
       ftypes = [ cl.file_type_pattern() for cl in file_type_class.values() ]
       ftypes.sort()
       for ftype in [any_file_type] + ftypes:
          tlist.insert(END, ftype)
       def select_file_type(y):
          index = int(tlist.nearest(y))
          if index < 0: return
          self.type_ent.delete(0, END)
          self.type_ent.insert(END, tlist.get(index))
          fr.destroy()
       tlist.bind('<Button-1>',
                  EventHandler('Select file type',
                               lambda event: select_file_type(event.y)))
       def history_motion(event):
          tlist.selection_clear(0, END)
          tlist.selection_set(int(tlist.nearest(event.y)))
       tlist.bind('<Motion>',
                  EventHandler('History list motion', history_motion, 0))
       fr.place(x=x, y=y, anchor=NE)
       def dismiss_if_out(event):
          win = event.widget
          if not str(win).startswith(str(fr)): fr.destroy()
       self.master.bind('<Button-1>', lambda event: dismiss_if_out(event))
#                        EventHandler('Dismiss pulldown list',
#                                     lambda event: dismiss_if_out(event), 0))
       ### will this binding get purged when listbox destroyed???


#-----------------------------------------

def log_file_menu_item(log_file_tuple):
    title = 'Pathena: %s' % log_file_tuple[0]
    path = os.path.join(log_dir, log_file_tuple[1])
    return log_file_tuple[0], lambda : view_log_file(title, path, END)

log_menu_items = ( log_file_menu_item(
                       ('File Indexing Log', indexing_log_name)),
                   log_file_menu_item(
                       ('Client Event Log', client_main_log)),
                   log_file_menu_item(
                       ('PostgreSQL Server Log', pg_server_log_name)),
#                   log_file_menu_item(
#                       ('Client Error Log', client_error_log)),
                   )

if explain_log:    # from explain_queries debug flag in app_config
    log_menu_items = log_menu_items + \
                     ( log_file_menu_item(
                           ('Query Explanation Log', 'query_explanations')), )

# This class specifies the menubar layout, then creates an instance of
# class GUI_elements, which constructs all of the key structures.

class Pathena_client(object):
    def __init__(self, master=None):
        menubar = (('File',
                     ('Submit Problem Report...',),
#                     ('Submit Problem Report', submit_problem_report),
                     None,
                     ('Exit  ',               self.exit, 'q'),
                     ('Exit and Stop Server', self.exit_stop),
                    ),
                   ('Edit',
                     ('Indexing Profiles...',   change_profiles),
                     ('Custom Source Types...', customize_source_types),
                     ('Preferences...',         change_preferences),
                    ),
                   ('View',
                     ('Database Status', checkServerStatus),
                     None,
                     ('Word Statistics (Path)',
                      lambda : showTsearchStats('path')),
                     ('Word Statistics (Content)',
                      lambda : showTsearchStats('content')),
                     None, ) +
                    log_menu_items ,
                   ('Indexing',
                     ('Index Files and Directories...', index_directories),
                     ('Clear Indexing Database...',     clear_database),
                     None,
                     ('Schedule Automatic Indexing...', schedule_indexing),
                     None,
                     ('Show Indexable File Types',      show_file_types),
                    ),
                   ('Help', 
                     ('Quick Start Guide',    quick_start),
                     ("User's Guide",         help_desk, 'h'),
                     ('About Pathena Client', about_window)))
        self.master = master
        self.gui = GUI_elements(master, self, menubar)
        master.protocol('WM_DELETE_WINDOW', self.exit)
        try:
            self.saveas_initialdir = os.environ['HOME']
        except:
            self.saveas_initialdir = '/'

    def exit(self):
        self.final_cleanup()
        self.master.after(100, lambda: self.master.destroy())

    def exit_stop(self):
        self.final_cleanup()
        stop_cmd = os.path.join(bin_dir, 'stop_server')
        os_system('(sleep 1; %s) &' % stop_cmd)
        self.master.after(100, lambda: self.master.destroy())

    def final_cleanup(self):
        # clean up any remaining temp files
        os.system('cd %s; rm -f %s_*' % (tmp_dir, os.getpid()))


#-----------------------------------------
                       
# Queries initiated from the GUI interface start their processing here.
# Argument query is a sequence of four strings: content, path, date, type.

def submit_query(query):
    global cur_cli
    q_parsed, results = perform_query(query, error_dialog_fn)
    if results == None: return
    query_string, num_results = ' ; '.join(query), len(results)
    emit_log_entry(main_log_file,
                   "Query terms '%s' resulted in %d files." %
                   (query_string, num_results))
    client.gui.query_history.insert(0, ('%04u' % num_results,) + tuple(query))
    if results == []:
        notify_empty_results(query)
        return
    query_cli = Query_Page(client, query)
    cur_cli = query_cli   ### debug only
    query_cli.gui.post_query_results(query, q_parsed, results)

# Rather than create a new tab/page with empty results, a popup window
# informs the user and then destroys itself after a delay.

def notify_empty_results(query):
    def make_window(fr):
        for font, text in \
            ((bold_font, 'The query'),
             (text_font, "    '%s'" % " ; ".join(query)),
             (bold_font, 'returned no matches.'),
             (small_text_font,
              "\n(This notice will disappear in a few seconds.)")) :
            row = Frame(fr)
            Message(row, font=font, text=text,
                    aspect=1500, justify=LEFT).pack(side=LEFT)
            row.pack(padx=10, pady=2, fill=X)
        fr.after(7000, lambda : fr.destroy())
    pop_up_toplevel_window(make_window, (), 'Pathena Query Results',
                           bottom_margin=5)


#-----------------------------------------

emit_log_header('Python Tk Client Main Log', main_log_file)  # sys.stdout)
emit_log_header('Python Tk Client Error Log', sys.stderr)

client = Pathena_client(mainwindow)

emit_log_entry(main_log_file,
               'New client session started.  Process ID %s, port %s.' %
               (os.getpid(), os.environ['PGPORT']))

# Start client with an initial query:

initial_query = \
    split_and_pad_terms(user_prefs.get('Client', 'initial_query'))
if initial_query:
    submit_query(initial_query)


mainwindow.mainloop()  # doesn't terminate until user closes main window

emit_log_entry(main_log_file, 'Client session terminated.')
