
# Includes classes and methods for displaying query results as well as
# setting up all GUI structures used to find search terms within file
# displays, select actions from context menus, do additional regular
# expression searches on text, etc.


import os, sys, string, glob
from time import strftime, localtime, asctime

from client_globals import *
from widget_utils   import *

from file_type_attribs import *
from db_operations     import *

from tabbed_notebook  import add_tabbed_page
from find_state       import Find_State, many_threshold
from command_services import *

from format_headlines import format_headlines
from file_display     import fill_file_display, starting_display_position


tag_margin             =  4
query_results_height   = 18
file_type_width        = 14
path_display_width     = 52
path_display_threshold =  3 + path_display_width
path_display_format    = '%s %4s  %-14s %s  %s'
                       # (cflag, flen, file_type, fdate, show_path)

path_view_size    = user_prefs.getint('Client', 'path_view_size')
content_view_size = user_prefs.getint('Client', 'content_view_size')


#-----------------------------------------

# This class provides an interface to the tabbed notebook features.
# It probably can be absorbed by the following class.

class Query_Page(object):
    def __init__(self, client, query_strings=('','','','')):
       def make_page(master):
          self.master = master
          self.gui = QP_GUI_elements(master, client, query_strings)
       # label priority: content, path, type, date
       tab_label = query_strings[0] or query_strings[1] or \
                   query_strings[3] or query_strings[2]
       add_tabbed_page(make_page, tab_label,
                       lambda : self.gui.clean_up())


# Create a window for query results, which will appear as a page in the
# tabbed notebook widget.  The basic layout is as follows:
#   1) a row of buttons and a display of the query terms
#   2) a text or listbox widget to show file items
#   3) a title bar with an entry widget for regexp searching
#   4) a text widget to show the file contents

class QP_GUI_elements(object):
    def __init__(self, master, app, query_strings):
        self.app=app
        self.search_terms = query_strings
        results_summary = Frame(master)
        button_width = 3       # keep narrow to fit on line

        self.qlines_count = Label(results_summary, text='0')
        self.qlines_count.pack(side=LEFT, padx=0)
        self.result_count = Label(results_summary, text='/ 0')
        self.result_count.pack(side=LEFT, padx=0)
        self.next_qlines = \
            ThinButton(results_summary, text='Next', width=button_width,
                       command=EventHandler('Next qlines button',
                                 lambda : self.format_qlines(self.view_size)))
        self.next_qlines.pack(side=LEFT, padx=3)
        self.all_qlines = \
            ThinButton(results_summary, text='All', width=button_width,
                       command=EventHandler('All qlines button',
                                            lambda : self.format_qlines(-1)))
        self.all_qlines.pack(side=LEFT, padx=0)
        self.create_query_box(results_summary, query_strings)
        reenter_query = \
            EventHandler('Reenter previous query text',
                         lambda: self.reenter_query(query_strings), 0)
        # need a way to keep following button from going off right end
        ThinButton(results_summary, text='Edit', width=button_width,
                   command=reenter_query).pack(side=RIGHT, padx=0)
        results_summary.pack(padx=5, pady=5, fill=X)
        self.results_summary = results_summary

        fr1 = Frame(master)
        self.create_query_control_panel(fr1, query_strings)
        fr1.pack(fill=X, padx=2)
        
#        Frame(master).pack(pady=1)
        fr2 = Frame(master)
        self.create_display_head(fr2)
        fr2.pack(padx=2, pady=2, fill=X)

        fr3, self.display = \
            scrolled_text_widget(master, width=70, height=30,
                                 font=text_font)
        self.display.configure(cursor='', padx=0, pady=0)  ###
        self.starting_display_position = starting_display_position
        fr3.pack(fill=BOTH, expand=YES, padx=2, pady=0)
        Frame(master).pack(pady=1)
        master.update_idletasks()

# A query display box is created to show the search terms that gave rise
# to the results below it.  Also included are handlers for bindings that
# display help information.

    def create_query_box(self, parent, query_strings):
        self.qbox = Label(parent, relief=SOLID, bd=1, anchor=W,
                          fg=colors['gray_text'],  pady=2,
                          background=colors['text_bg_gray'],
                          text=' %s  ;  %s  ;  %s  ;  %s ' %
                               tuple(query_strings))
        self.qbox.bind('<Enter>', self.qbox_enter_action)
        self.qbox.bind('<Leave>', self.qbox_leave_action)
        self.qbox_action_id = None
        self.qbox_text_orig = self.qbox.cget('text')
        self.qbox.pack(side=LEFT, padx=7, expand=YES, fill=X)

    def qbox_enter_action(self, *args):
        def show_context_menu_message():
            self.qbox_action_id = 1
            self.qbox.config(text='Right click for context menu on path list',
                             fg='red4')
        self.qbox_action_id = \
            mainwindow.after(2000, show_context_menu_message)

    def qbox_leave_action(self, *args):
        if self.qbox_action_id == 1:
            self.qbox.config(text=self.qbox_text_orig, fg=colors['gray_text'])
        elif self.qbox_action_id:
            mainwindow.after_cancel(self.qbox_action_id)
        self.qbox_action_id = None
        
# When the Edit button is selected, fill the main query entries with values
# from the current results page.  Briefly highlight the text and its origin.

    def reenter_query(self, query_strings):
        highlights = self.app.gui.q_entries
        def reset_text_color(highlight_widgets):
            for w in highlight_widgets:
                w['fg'] = colors['text_fg_color']
            self.qbox['fg'] = colors['gray_text']
        for ent, q_str in zip(self.app.gui.q_entries, query_strings):
            ent.delete(0, END)
            ent.insert(END, q_str)
        for w in highlights + [self.qbox]:
            w['fg'] = 'red3'
        mainwindow.after(5000, reset_text_color, highlights)

# Both the text display widget and the listbox showing file items have
# the same context menu.

    def create_display_context_menu(self):
        display_popup_handler = \
            self.common_context_popup(self.content_view, 1)
        def pop_up(event):
            self.popup_index = self.selected_index
            display_popup_handler(event)
        self.display.bind('<Button-3>',
                          EventHandler('Display popup context menu',
                                       pop_up, 0))

# A separate context menu is used for the entire path list.  Path names
# can be written to a file or exported to an external editor.

    def path_list_popup(self):
        def copy_path_names(extract_fn, unique):
            path_names = [ extract_fn(p[1]) for p in self.path_list ]
            if unique: path_names = unique_list(path_names)
            copy_text_clipboard(' '.join(path_names))
        def start_term(cmd, ex_opt):
            path_names = [ "'%s'" % escaped_blanks(p[1], 1)
                           for p in self.path_list ]
            invoke_terminal('cd ~', cmd, ex_opt, *path_names)
        terminal_menu = \
            [ (t[0], lambda cmd=t[1],ex_opt=t[2]: start_term(cmd, ex_opt))
              for t in terminal_windows ]
        choices = [ ('Copy Search Terms',
                     lambda : copy_text_clipboard(
                                  ' ; '.join(self.search_terms))),
                    None,
                    ('Copy Full Path Names',
                     lambda : copy_path_names(lambda x: x, 0)),
                    ('Copy All Directory Paths',
                     lambda : copy_path_names(os.path.dirname, 0)),
                    ('Copy Unique Directory Paths',
                     lambda : copy_path_names(os.path.dirname, 1)),
                    ('Copy All File Names',
                     lambda : copy_path_names(os.path.basename, 0)),
                    ('Copy Unique File Names',
                     lambda : copy_path_names(os.path.basename, 1)),
                    None,
                    ('Save Path Names As...',
                     lambda : self.write_paths_to_file(
                                  [ p[1] for p in self.path_list ])),
                    None,
                    ('Invoke Shell Command on Paths...',
                     lambda : invoke_cmd_on_path_list(
                                  self.write_paths_to_file,
                                  [ p[1] for p in self.path_list ])),
                    ('Open Terminal Window on Paths', terminal_menu),
                    ('Explain Terminal Window on Paths',
                     self.explain_terminal_on_paths),  ]
        menu = add_popup_menu(self.path_widget, choices)
        def display_popup(event):
            menu.tk_popup(*menu.winfo_pointerxy())
        return display_popup

    def write_paths_to_file(self, path_list):
        file_name = asksaveasfilename(title='Save Path Names As...',
                                      initialdir=self.app.saveas_initialdir)
        if file_name:
            self.app.saveas_initialdir = os.path.dirname(file_name)
            path_names = [ p+'\n' for p in path_list ]
            write_text_file(path_names, file_name, 1, error_dialog_fn)

    def explain_terminal_on_paths(self):
        def make_window(fr):
            Message(fr, aspect=250, text=display_messages['explain_terminal'],
                    font=text_font).pack(padx=10)
            button_row(fr, (('Close', lambda : fr.destroy(), 0), )
                       )[0].pack(pady=10)
        pop_up_toplevel_window(make_window, (),
                               'Explain Terminal Window on Paths',
                               bottom_margin=0)
        

# Two possible subpanels can be created according to query type.
# If content search terms are present, use the content-based panel;
# otherwise, use the path-based panel.

    def create_query_control_panel(self, win, query_strings):
        if query_strings[0]:
            self.content_query_control_panel(win)
        else:
            self.path_query_control_panel(win)
        qbox_popup_handler = self.path_list_popup()
        self.qbox.bind('<Button-3>',
                       EventHandler('Path list popup context menu',
                                    qbox_popup_handler, 0))

# The path-based panel is a scrolled listbox showing file items.

    def path_query_control_panel(self, win):
        fr, self.path_widget = \
            scrolled_listbox(win, height=query_results_height, width=60,
                             font=small_fixed_font) #  horiz_scroll=1,
        self.path_widget.configure(cursor='')
        def get_file_contents(win, y):
            index = win.nearest(y)
            if index >= 0:
                self.view_file(0, 1, 1, index)
                self.selected_index = index
        self.path_widget.bind('<Button-1>',
            EventHandler('Path control path_widget selection',
                         lambda event:
                             get_file_contents(event.widget, event.y)))
        self.popup_handler = self.common_context_popup(0, 1)
        def pop_up(event):
            self.popup_index = self.path_widget.nearest(event.y)
            self.popup_handler(event)
        self.path_widget.bind('<Button-3>',
                              EventHandler('Path popup context menu',
                                           pop_up, 0))
        fr.pack(fill=BOTH, expand=YES)

# A content-based panel is a scrolled text widget with a lot of tagged
# regions created by functions in module format_headlines.

    def content_query_control_panel(self, win):
        fr, self.path_widget = \
            scrolled_text_widget(win, width=60, height=query_results_height)
        self.path_widget.config(padx=0, pady=0)  ###
        fr.pack(fill=BOTH, expand=YES)
        self.popup_handler = self.common_context_popup(1, 0)
        self.path_widget.configure(cursor='')
        ### following don't seem to work:
        self.path_widget.unbind('<Double-Button-1>')
        self.path_widget.unbind('<Triple-Button-1>')


# A regexp-based text search feature is implemented to search for strings
# in the display window.  Args to this function indicate behavior:
#  next = 0 for starting at beg/end of file, = 1 for search from prev match;
#  dir  = 1 for forward, = -1 for backward;

    def find_in_display(self, event, next=0, dir=1):
        # event is a dummy arg for absorbing Tkinter event objects
        pattern = self.find_entry.get()
        find_occur = self.find_state[self.selected_index]
        if not next and pattern == find_occur.prev_find_pattern: return
        self.display.tag_delete('highlight_line', 'find_match')
        self.display.see(1.0)
        if next and find_occur.prev_occ_values[1][2]:
            # previously found occurrences exist
            self.find_in_display_next(dir, find_occur)
        else:
            self.find_in_display_new(pattern, find_occur)

# Advance to the next/previous occurrence.

    def find_in_display_next(self, dir, find_occur):
        occurrence, total = find_occur.prev_occ_values[1][1:]
        occurrence += dir                         # 1-base index
        if not (1 <= occurrence <= total):
            if total:
                occurrence -= total * dir         # dir = +/- 1
            else:
                find_occur.show_occurrence(1, dir, '-----  ')
                if dir == 1: self.display.see(END)
                else:        self.display.see(1.0)
                return
        find_occur.prev_occ_values[1][1] = occurrence
        find_occur.show_occurrence(1, dir)
        site_beg_end = find_occur.find_sites[occurrence - 1]
        self.display.tag_add('find_match', *site_beg_end)
        find_occur.highlight_line(float(site_beg_end[0]), 'find_match')

# For a new pattern, do some setup then display the first occurrence.

    def find_in_display_new(self, pattern, find_occur):
        disp = self.display
        count_var = IntVar()
        find_occur.find_sites = []
        start = self.starting_display_position
        try:
            match_beg = pattern and \
                        disp.search(pattern, start, END, regexp=1, nocase=1,
                                    count=count_var)
        except:   # allow temporarily ill-formed pattern, but alert user
            self.find_entry.configure(background=colors['find_error_bg'])
            find_occur.show_occurrence(1, 1, '?????  ', 0)
            return

        if pattern and has_regexp_metachars(pattern):
            self.find_entry.configure(background=colors['headline_select'])
        else:
            self.find_entry.configure(background=colors['text_bg_color'])

        total_count = 0
        while match_beg:
            match_end = disp.index('%s + %s chars' %
                                   (match_beg, count_var.get()))
            find_occur.find_sites.append((match_beg, match_end))
            total_count += 1
            if total_count >= many_threshold: break
            match_beg = disp.search(pattern, match_end, END, nocase=1,
                                    count=count_var, regexp=1)
        find_occur.prev_occ_values[1] = [1, 0, total_count]
        if pattern:
            self.find_in_display_next(1, find_occur)
        else:
            find_occur.clear_occurrence()
        find_occur.prev_index = 1


# Place just above the text display area a title and the widgets needed
# for the auxiliary text search feature (find function).

    def create_display_head(self, header):
        fr1, fr2 = Frame(header), Frame(header)
        self.path_label = Message(fr1, width=400, justify=LEFT)
        self.path_label.pack(side=LEFT)
        self.find_entry = Entry(fr1, highlightthickness=0,
                                width=20, background=colors['text_bg_color'])
        wbut_size = 13
        wbut_padx = (wbut_size - 9) // 2
        for dir, arrow, key in ((1, u_triagrt, 's'), (-1, u_triaglf, 'r')):
            find_next = EventHandler('Find next match button',
                                     lambda ev=0, dir=dir:
                                         self.find_in_display(ev, 1, dir), 0)
            wedge = Button(fr2, text=arrow, padx=1, pady=0,
                           command=find_next)
            wedge.pack(side=RIGHT, padx=wbut_padx)
            self.find_entry.bind('<Control-%s>' % key, find_next)
        self.find_entry.pack(side=RIGHT, padx=2)
        
        # modify binding order so entry is updated before handler runs
        find_entry_id = self.find_entry.winfo_parent() + '.' + \
                        self.find_entry.winfo_name()
        self.find_entry.bindtags(('Entry', find_entry_id, '.', 'all'))
        self.add_find_context(fr1)
        Label(fr1, text=' Find:').pack(side=RIGHT, padx=0)
        # filter out shift keys, control keys, etc.
        def key_find(event):
            if event.char: self.find_in_display(event)
        # make search incremental (update on every keystroke):
        self.find_entry.bind('<Key>',
                             EventHandler('Find key entry', key_find, 0))
        fr2.pack(side=RIGHT)
        fr1.pack(side=LEFT, expand=YES, fill=X)

# Add context menu and handlers for dynamic Find entry messaging.

    def add_find_context(self, header):
        header.bind('<Enter>', self.find_entry_enter_action)
        header.bind('<Leave>', self.find_entry_leave_action)
        self.find_entry_action_id = None
        self.find_entry_text_orig = ''
        self.find_entry_bf_orig   = colors['text_bg_color']
        self.find_entry.bind('<Button-3>',
                             EventHandler('Display find popup context menu',
                                          self.find_context_popup(), 0))
        self.find_entry_header = header

# The actual popup menu is constructed here.  It tries to include various
# combinations of the query search terms as possible patterns.

    def find_context_popup(self):
        def clear_insert(regexp=''):
            self.find_entry.delete(0,END)
            self.find_entry.insert(END, regexp)
            self.find_in_display(None)   # as if entered by keystrokes
        def make_choice(combo):
            esc_combo = '.*'.join(map(regexp_escape, combo))   ## re.escape
            return ("Insert '%s'" % esc_combo,
                    lambda esc_combo=esc_combo: clear_insert(esc_combo))
        terms = self.app.gui.q_entries[0].get().split()
        # don't want outer quotes on terms for regexps
        terms = [ t.strip('"\'') for t in terms ]
        if len(terms) > 1:
            combos = [ (a,b) for a,b in zip(terms[:-1], terms[1:]) ]
        elif terms:
            combos = [terms]
        else:
            combos = []
        if len(terms) > 2: combos.append(terms)
        if combos:
            combo_choices = [ make_choice(c) for c in combos ]
        else:
            combo_choices = []  ### something for null case???
        choices = [ ('Clear Regular Expression', lambda : clear_insert()),
                    None ] + combo_choices
        menu = add_popup_menu(self.find_entry, choices)
        def display_popup(event):
            menu.tk_popup(*menu.winfo_pointerxy())
        return display_popup

# Entering the title bar can trigger a baloon-style help message about Find.

    def find_entry_enter_action(self, *args):
        def show_context_menu_message():
            self.find_entry_action_id = None
            balloon = Label(self.find_entry_header, relief=SOLID, bd=1,
                            text=' Right click in Find box for menu ',
                            bg=colors['headline_select'])
            balloon.place(anchor=CENTER, relx=0.5, rely=0.5)
            mainwindow.after(5000, lambda : balloon.destroy())
        if self.find_entry_action_id == None:
            self.find_entry_action_id = \
                mainwindow.after(5000, show_context_menu_message)

    def find_entry_leave_action(self, *args):
        if self.find_entry_action_id != None:
            mainwindow.after_cancel(self.find_entry_action_id)
        self.find_entry_action_id = None


# A context menu is available for each listbox or headline item, allowing
# the user to perform various actions based on path.

    def common_context_popup(self, content_view, force_display):
        def invoke_editor(command, index):
            abs_path = self.path_list[index][1]
            os_system(command % abs_path, redirect=2, background=1)
        self.editor_menu = \
            [ (e[0], lambda cmd=e[1]: invoke_editor(cmd, self.popup_index))
              for e in text_editors ]
        def start_term(command, exec_option, index):
            abs_path = self.path_list[index][1]
            if not os.path.isdir(abs_path):
                abs_path = os.path.dirname(abs_path)
            invoke_terminal('cd "%s"' % abs_path, command, exec_option)
        terminal_menu = \
            [ (t[0], lambda cmd=t[1],ex_opt=t[2]:
                         start_term(cmd, ex_opt, self.popup_index))
              for t in terminal_windows ]
        choices = [ ('', null_proc, 'path_label'),
                    None,
                    ('Display File in Separate Window',
                     lambda : self.view_file(
                                  content_view, 0, 1, self.popup_index,
                                  force_display=force_display),
                     'separate'),
                    ('Display File in Lower Panel',
                     lambda : self.view_file(
                                  content_view, 1, 1, self.popup_index,
                                  force_display=force_display),
                     'panel'),
                    None,
                    ('Copy Full Path Name',
                     lambda : copy_text_clipboard(
                                  self.path_list[self.popup_index][1]),
                     'full_path'),
                    ('Copy Directory Path',
                     lambda : copy_text_clipboard(
                                  os.path.dirname(
                                      self.path_list[self.popup_index][1])),
                     'dir_path'),
                    ('Copy File Name',
                     lambda : copy_text_clipboard(
                                  os.path.basename(
                                      self.path_list[self.popup_index][1])),
                     'file_name'),
                    None,
                    ('Launch External Editor on File', self.editor_menu,
                     'editor'),
                    None,
                    ('Invoke Shell Command on File...',
                     lambda : invoke_cmd_on_path(0,
                                  self.path_list[self.popup_index][1]),
                     'shell_file'),
                    ('Invoke Shell Command on Directory...',
                     lambda : invoke_cmd_on_path(1,
                                  os.path.dirname(
                                      self.path_list[self.popup_index][1])),
                     'shell_dir'),
                    ('Open Terminal Window on Directory', terminal_menu,
                     'terminal'), ]
        menu, tag_dict = add_popup_menu(self.path_widget, choices)
#        menu.entryconfigure(tag_dict['path_label'],
#                            foreground=colors['muted_red'])
        menu.entryconfigure(tag_dict['path_label'], state=DISABLED)
        init_enabled_tags = tag_dict.keys()
        init_enabled_tags.remove('path_label')
        init_enabled = [ tag_dict[t] for t in init_enabled_tags ]
        def display_popup(event):
            if self.popup_index < 0: return      # empty listbox
            for i in init_enabled: menu.entryconfigure(i, state=NORMAL)
            file_type = self.path_list[self.popup_index][2]
            file_class = file_type_class[file_type]
            attrib = file_class.file_indexing_attrib
            disable = []
#            if attrib == 'encoded':      disable.append('separate')
#            elif attrib == 'path_list':  disable.append('shell_file')
            if not file_class.external_editors: disable.append('editor')
            if self.popup_index != self.selected_index: disable.append('panel')
            for d in disable: menu.entryconfigure(tag_dict[d], state=DISABLED)
            path_name = self.path_list[self.popup_index][1]
            if len(path_name) > 40: path_name = '...'+path_name[-38:]
            menu.entryconfigure(tag_dict['path_label'],
                                label='    %s' % path_name)
#                                label='Path: %s' % path_name)
            menu.tk_popup(*menu.winfo_pointerxy())
        return display_popup


# After running a query and creating a new results page, the main GUI
# event handler invokes this procedure to format and post the results.
# Query terms of form: (content terms, path terms, date terms, file type)

    def post_query_results(self, queries, q_parsed, results):
        self.result_count.config(text='/ %d' % len(results))
        self.queryResults = results
        # Returns a list of tuples according to category:
        #   path query:    [file_id, abs_path, file_type, content_vector_len]
        #   content query: [file_id, abs_path, file_type, rank]
        self.path_list = results[:]  ### why is copy needed???
        self.selected_index = None
        self.sites = []
        self.displayed_before = [ None for p in self.path_list ]
        self.find_state       = [ None for p in self.path_list ]
        self.find_occurrence = None
        self.after_display_action = None

        self.num_qlines = 0
        self.content_view = queries[0]
        self.create_display_context_menu()
        # Results is a list of tuples/lists
        if self.content_view:                 # content query results
            self.view_size = content_view_size
            self.format_qlines = lambda size: \
                format_headlines(self, self.path_list, q_parsed[0], size)
        else:
            self.view_size = path_view_size
            self.format_qlines = lambda size: \
                self.format_pathlines(self.path_list, size)
        self.format_qlines(self.view_size)

    def adjust_continue_buttons(self):
        if self.num_qlines == len(self.path_list):
            self.next_qlines.configure(state=DISABLED)
            self.all_qlines.configure(state=DISABLED)

# Format path descriptor items after query results are received.
# This is a simple display containing one listbox entry per file
# showing the path name and several attributes.  Selecting a listbox
# item causes the file to be displayed.
                            
    def format_pathlines(self, items, num):
        start = self.num_qlines
        if num == -1: num = len(items) - start   # do the rest
        for id, abs_path, file_type, content in items[start:start+num]:
            if os.path.exists(abs_path):
                size = os.path.getsize(abs_path)
                flen = approx_size(size, 1)
                fdate = strftime('%b %d %Y',
                                 localtime(os.path.getmtime(abs_path)))
            else:
                flen, fdate = '----', 'Deleted    '
            if abs_path.startswith(user_path_prefix):
                show_path = '~/' + abs_path[user_path_prefix_len:]
            else:
                show_path = abs_path
            if len(show_path) > path_display_threshold:
                show_path = '...' + show_path[-path_display_width:]
            if content == None: cflag = ' '
            else:               cflag = 'c'
            self.path_widget.insert(END, path_display_format %
                                    (cflag, flen, file_type, fdate, show_path))
        # change to text widget, add term highlighting
        self.path_widget.see(start)
        self.num_qlines = min(start + num, len(items))
        self.qlines_count.config(text='%d' % self.num_qlines)
        if start < len(items):
            self.view_file(0, 1, 1, start)
            self.selected_index = start
            self.path_widget.selection_clear(0, END)
            self.path_widget.selection_set(start)
        self.adjust_continue_buttons()

# Following is used to insert file text as appropriate in the display
# window (lower panel).  Most of the real work is performed in
# fill_file_display.

    def view_file(self, content_view, attached, fetch_needed, item_num,
                  force_display=0, title_text=None):
        nonexistent = 0
        if fetch_needed:
            try:
                fill_file_display(self, content_view, attached, item_num,
                                  force_display, title_text)
            except DeletedFileError:
                nonexistent = 1
        if not attached: return
        if self.selected_index != None and \
           self.find_state[self.selected_index]:
            self.find_state[self.selected_index
                            ].save_occurrence_state(content_view)
        if nonexistent: return
        mod_time = os.path.getmtime(self.path_list[item_num][1])
        if not content_view:
            self.restore_find_state(0, fetch_needed, item_num, mod_time)
            return
        if not self.find_state[item_num]:
            # self.sites should be set by fill_file_display
            # set site indices to -1 so first increment goes to instance 0
            self.cur_site = [ -1 for s in self.sites ]
            self.hterm_total_count =    [ len(s) for s in self.sites ]
        self.restore_find_state(1, fetch_needed, item_num, mod_time)
        if self.after_display_action:
            self.after_display_action()
            self.after_display_action = None

    def restore_find_state(self, content_view, fetch_needed, index, mod_time):
        def set_new_pattern():
            self.find_entry.config(bg=colors['text_bg_color'])
            self.find_entry.delete(0, END)
            self.find_entry.insert(END, self.find_entry.get())
            self.find_state[index].prev_occ_values[1] = [1, 0, 0]
#        file_changed = self.find_state[index] and \
#                       (mod_time != self.find_state[index].file_mtime)
#        if self.find_state[index] and not file_changed:
        if self.find_state[index]:
            if fetch_needed: 
                self.find_state[index].restore_occurrence_state(content_view,
                                                                mod_time)
                if not self.find_entry.get(): set_new_pattern()
        else:
            # following unmaps occurrence count window as a side effect
            self.find_state[index] = \
                Find_State(self, self.display, content_view, mod_time)
            set_new_pattern()
#            if file_changed:
#                self.find_state[index].show_file_changed()

# Perform cleanup of resources when this tabbed page is deleted.

    def clean_up(self):
        for temp_file in self.displayed_before:
            if temp_file:
                try:    os.remove(temp_file)
                except: pass


##################

# Key variables need to maintain the following invariant, which is
# checked after every action if enabled by check_assertions flag.
# The assertion is not too thorough; basically it ensures
# that data items in certain instance attributes have suitable values.
# This checking is still less than full consistency.

    def state_invariant(self):
        return self.state_invariant_details()

    def state_invariant_details(self):
        if self.selected_index != None and \
           self.selected_index >= len(self.path_list):
            return 'selected_index out of range: %s\n\n' % self.selected_index
        
        for i in range(len(self.find_state)):
            s = self.find_state[i]
            if s != None and not isinstance(s, Find_State):
                return 'find_state[%d] invalid type: %s\n\n' % (i, s)
            
        if self.after_display_action != None:
            return 'after_display_action: %s' % self.after_display_action

        if self.selected_index != None and \
           self.find_state[self.selected_index]:
            fs = self.find_state[self.selected_index]
            if fs.currently_showing and len(fs.currently_showing) < 4:
                return 'currently_showing: %s' % fs.currently_showing

        return 0    # no problems detected

##################
