# Each time the client is started, a check is made to see if the
# user preference file is missing any options.  This can happen
# when upgrading to a new version.  Any missing options are added
# automatically to the file (with annotation).

import os, re

from ConfigParser import ConfigParser

from app_globals import ConfigParserCommentless
    

def update_user_prefs(user_prefs_file, distrib_prefs_file, cur_version):
    global _current_config
    user_config = ConfigParserCommentless()
    user_config.read([user_prefs_file])
    update_needed = 1
    if user_config.has_option('Version', 'prefs_version'):
        if cur_version == user_config.get('Version', 'prefs_version'):
            update_needed = 0
    if update_needed:
        merge_config_files(user_prefs_file, distrib_prefs_file,
                           user_config, cur_version)
#    else:
#        return user_config
#    final_config = ConfigParserCommentless()
#    final_config.read([user_prefs_file])
#    _current_config = final_config
#    return final_config

def merge_config_files(user_prefs_file, distrib_prefs_file,
                       user_config, cur_version):
    # following shouldn't use Commentless class; leave comments intact
    distrib_config = ConfigParser()
    distrib_config.read([distrib_prefs_file])
    settings = []
    for sect in user_config.sections():
        user_items = user_config.items(sect)
        settings.extend([ (sect, opt, val) for opt, val in user_items
                          if val != distrib_config.get(sect, opt) ])
    # override old version number with current
    settings.append(('Version', 'prefs_version', cur_version))
    conf = open(distrib_prefs_file)
    lines = conf.readlines()
    conf.close()
    for sect, opt, val in settings:
        line_num = 0
        sect_pattern = re.compile('\[%s\]' % sect)
        while not sect_pattern.match(lines[line_num]): line_num += 1
        opt_pattern = re.compile('%s\s*(:|=)\s*(\S.*)' % opt)
        while not opt_pattern.match(lines[line_num]): line_num += 1
        opt_line = lines[line_num]
        start = opt_pattern.match(opt_line).start(2)
        lines[line_num] = opt_line[0:start] + val + '\n'
    conf = open(user_prefs_file, 'w')
    conf.writelines(lines)
    conf.close()
    

