# Text sections of User's Guide

import os
#from Tkinter import BitmapImage

helpItems = (
    
('Introduction' ,
"Welcome to Pathena Desktop Search.  Pathena is an application for \
indexing files and directories on desktops, currently limited to Unix \
family operating systems. Search features implemented in \
this client module enable users to find files based on path name or \
file content. User-defined profiles guide the discovery of relevant \
items within the user's file system. \
\n\n\
Several desktop search tools exist for various platforms, some as \
commercial products and others as free or open source applications. \
More than most of these offerings, Pathena was designed with \
technically oriented users in mind, particularly those well acquainted \
with the concept of a hierarchical file system.  Pathena is an open \
source tool distributed under a BSD-style license. \
\n\n\
Select a topic from the list on the left to \
view a description of the corresponding features. \
If you are a new user, begin with the Overview. \
Consult the more detailed topics as needed. \
\n\n\
The main Pathena web site is located at: \
\n\n    http://TBD/pathena.html \
\n\n\
Pathena's developers can be reached at: \
\n\n    pathena@TBD \
\n\n\
" ) ,

('Overview' ,
"Pathena makes use of full-text indexing and search techniques. \
Its software architecture is based on the following components: \
\n\n\
    Server -- implemented using the PostgreSQL database management system, \
including the Tsearch2 module for full-text indexing. The server runs \
quietly as a background process on your desktop machine. \
\n\n\
    Client -- implemented as Python software using the Tk GUI toolkit. \
The client provides the primary interface, allowing users to invoke \
queries and display results. Additionally, users may view file contents, \
launch external viewers, invoke shell commands, and perform other tasks \
based on query results. \
\n\n\
    Scripts -- includes assorted items for conducting file indexing and \
performing various maintenance chores. \
\n\n\
Currently Pathena requires that file indexing be performed as a separate, \
explicitly managed activity. \
Users specify indexing profiles to indicate which portions of \
their file systems are subject to indexing and which file types to include. \
Indexing can be performed on demand or in a regularly schedule manner. \
Future versions will aim for tighter integration with the \
underlying file system to reduce the indexing lag. \
\n\n\
Pathena is designed to index individual files \
as maintained by the operating system of a desktop or workstation. \
It is not intended to access application-specific data stored in \
proprietary database formats. Nor does it attempt to index \
your browser histories or track other types of recent access patterns. \
It works best when indexing data with a certain amount of permanence \
rather than ephemeral information. \
Also, Pathena's operation is strictly local; it makes no attempt to access \
any Internet sites or initiate external communication of any kind. \
" ) ,

('File Indexing' ,
 "Indexing may be conducted based on profiles stored in subdirectory \
~/.pathena/profiles. \
Several profiles come with the distribution: \
\n\n\
    test -- indexes the Pathena distribution in ~/.pathena \n\
    all_paths -- does path-only indexing of (nearly) all user files. \n\
    profile_template -- used to create new profiles \n\
\n\
To create a custom profile P, first copy the file 'profile_template' to P, \
then edit P as needed using an ordinary text editor.  Alternatively, the \
Edit menu has an item for editing profiles. Select 'profile_template' first, \
edit as desired, then invoke Save Profile As... to write it back under \
a new name. \
See the file 'template_howto' for an explanation of profile conventions. \
\n\n\
Any number of profiles may be used for file indexing. All indexing \
information that results will be merged into the same database. \
\n\n\
Indexing may be initiated from the GUI client or run via a batch script. \
New users typically will start by building a few small profiles then \
launch them from the GUI during their initial experimentation period. \
Once a set of production profiles has been established, the suggested \
way to maintain the database is to run a nightly 'cron' job to reindex \
the files in the user's profiles. A menu item is provided to help set up \
the 'crontab' entries needed to carry this out. Using this setup, the \
indexes will be refreshed nightly without any special effort on the \
user's part. \
" ) ,

('Supported File Types' ,
 "Indexing of file content may only be performed for certain file types. \
The following list itemizes what types are supported by the current \
implementation. This list will grow as Pathena's repertoire is expanded. \
Currently available types may be viewed by selecting the menu item \
Indexing > Show Indexable File Types. \
\n\n\
File type names have the form <genus>/<species>. Patterns are used to \
select subsets of type names. For example, 'doc/.*' refers to all \
document types. \
File types fall into three broad categories: regular, special and \
implicit. \
\n\n\
Regular File Types \
\n\n\
    text/.* -- includes textual files of many varieties that comprise \
characters drawn from standard character sets such as ASCII, Latin-1 \
or Unicode. Includes plain text files (.txt) and those having no \
file extension.\
\n\n\
    source/.* -- includes files for source code in a variety of languages. \
The syntax for identifiers in each language is accommodated. \
Complete identifiers and their subwords are both indexed. \
Users may extend this genus with custom source types. \
\n\n\
    doc/.* -- for documents in encoded binary or textual formats \
that require conversion to plain text form before indexing. \
Includes species pdf, postscript, msword, oowriter, etc. \
\n\n\
    markup/.* -- Textual files containing embedded markup notation \
appear in this genus. Included are species html, xml and tex. \
\n\n\
    archive/.* -- Encoded (and possibly compressed) archive formats \
are assigned to this genus. Only the path names of the archived files \
appear in the indexed content. Included species are tar, zip and rpm. \
\n\n\
Special File Types \
\n\n\
    email/maildir -- a standard e-mail message/mailbox format that \
places each message in a \
separate file having an assigned name within a prescribed directory \
structure. Messages are text files having stylized headers and \
MIME notation. \
\n\n\
Implicit File Types \
\n\n\
    node/direc -- every directory within the profile-specified subtrees of \
the file system will have its path indexed. \
Furthermore, a directory is considered to have indexable \
'content', namely, the file names of its children. \
\n\n\
    node/link -- symbolic links (similar to node/direc). \
\n\n\
    exec/script -- executable text files written in a known scripting \
language are treated similar to source file types. \
\n\n\
    exec/binary -- binary executables have no indexable content. \
\n\n\
    other/misc -- files whose extensions are not supported will be assigned \
species 'misc', meaning that content indexing cannot be performed. \
\n\n\
    other/no_ext -- nontext files lacking extensions will be assigned \
this type, meaning that content indexing cannot be performed. \
\n\n\
    other/oversize -- huge files that otherwise would be eligible \
for content indexing but \
are regarded as too large will be assigned to species 'oversize'. \
\n\n\
" ) ,

('Indexing E-Mail' ,
 "On Unix systems, two main conventions for storing messages are \
in common use, the so-called 'mbox' and 'maildir' formats. In the 'mbox' \
format, all messages of a given mailbox are stored in a single file, \
with each new message beginning with the 'From' line. Pathena does not \
support this type of e-mail storage. \
\n\n\
The 'maildir' format, on the other hand, is supported and \
is much more consistent with Pathena's mode of operation. In the 'maildir' \
format, a mailbox is organized into a trio of subdirectories \
called 'cur', 'new' and 'tmp', where all received messages settle into \
'cur'. Each message is stored in its own file with a numerical file name \
that is meaningless to users. The importance of this structure, though, \
is that content-based searching will identify an individual file and  \
retrieve its corresponding message for display to the user. \
\n\n\
Mailboxes in mbox format can be converted \
to maildir format for e-mail clients that support it. \
A Perl script called 'mb2md.pl' can be downloaded \
to automate this conversion. \
" ) ,

('Search Terms' ,
 "To query the Pathena file database, enter one or more search terms in \
the appropriate entry fields of the client GUI. \
Matches must satisfy all terms simultaneously. \
\n\n\
Types of search terms include: \
\n\n\
 - Text: words found within a file's textual content (after conversion); \
uses full-text search with stemming and stop words; \
\n\n\
 - Path: 'words' found within a path name, or file name pattern(s) \
formed using regular expressions; \
\n\n\
 - Date: one or two date-time strings, each of the form y/m/d/h/m/s, \
truncated at any point; if omitted, the second date is 'now'; \
\n\n\
 - Type: file type (or pattern) as specified by indexing profiles. \
\n\n\
Leaving a field blank means there will be no constraints of that kind \
in the search criteria. Any combination of the four fields may be used to \
express a query. \
\n\n\
For the path and content search terms, each 'word' is an alphanumeric \
string containing at least one letter. Special characters may also be \
used. Some 'words' containing embedded blanks (e.g., file names) may be \
specified by enclosing them in quotes (single or double). \
File extensions may be searched by providing a leading period. \
Path terms may include regular expressions using the forms \
re'<pattern>' or re\"<pattern>\". \
\n\n\
Search terms may be expressed in negative form by preceding them with \
'!' or '-' (without intervening blanks). Negative terms indicate that \
successful matches must not contain those words. \
\n\n\
Queries can also be entered using a single-string format. Terms for the four \
categories are written with semicolons as delimiters. Typing terms \
in the Text entry using this format will cause them to be parsed \
appropriately. Semicolon-delimited queries are also \
used with the command line script, ptfind. \
\n\n\
" ) ,

('Query Results',
 "After the client receives the server's response to a query request, \
a new tabbed page is created to display the results. \
At the top are the search terms for future reference. The corresponding Edit \
button allows those terms to be reinserted into the query entry fields for \
editing and resubmission. The upper left corner displays numbers of the \
form 'M / N', which means the first M cases out of a total of N matches \
are displayed below. \
\n\n\
A history list is maintained of all queries (search term sets) that \
were issued during the current session. You may see this list \
(displayed in reverse chronological order) by \
clicking on the Redo button. \
Selecting an item resubmits its corresponding query. \
\n\n\
Two types of file list display are provided. If no content search terms \
are present, the file list contains one line per file with each showing \
the size, type, date and path name for the file. \
A 'c' at the beginning of a line means the file's content has been indexed. \
\n\n\
In the second case, when \
content search terms are present, the file list shows a multiline entry \
for each matching file. The first line displays either the file name or an \
appropriate title if such is available for the document stored in that \
file. The bottom line shows the file's ranking as given by the full-text \
search algorithm, then the file's type, size, date and path name. The \
remaining lines show a text sample \
with search terms given in context. Each highlighted term acts as a button; \
clicking on it advances the display of the file in the window below. \
\n\n\
Context menus (reached by right-clicking) are provided for file list items. \
Clicking with the right button on a file item pops up a menu for displaying \
the file in a separate window, copying the path/file name to the clipboard, \
or launching an external editor on the file. \
\n\n\
At the top of each tabbed page, the search-term display \
has a context menu that enables you to copy to the clipboard \
the path/file names for all items that \
satisfied the query (the list could be long). You may also invoke \
shell commands and launch terminal windows from context menus. \
This is useful for second-stage processing tasks such as running grep \
on the file list returned by a query. \
\n\n\
" ) ,

('Secondary Text Search',
 "After displaying the text form of a file, you might care to do \
additional searches within that text. Pathena offers regular \
expression string searching for this purpose. Sitting above \
the display panel on the right is a text entry field labeled 'Find'. \
Typing a search string in this area invokes pattern matching \
as each character is typed. The behavior is similar to that of \
incremental search in Emacs (ctrl-s). \
\n\n\
After each character is typed, a small display in the upper right corner \
is updated to show occurrence count information \
and the file display advances to the first matching occurrence. \
You may stop at any point to visit other occurrences using the \
navigation buttons. Adding to your search string will resume the \
matching process. \
\n\n\
Pattern matching is implemented using Tk's built-in regular expression \
engine, which supports POSIX-style regular expressions (with minor \
variations). When you type a meta-character such as '.' or '*', the \
background will turn yellow as a reminder that what you're typing might \
not match text literally. For example, the pattern 'self.path' \
matches more strings than 'self.path' taken literally. \
For the more restricted search, you need to type 'self\\.path'. \
\n\n\
When an invalid pattern is detected, a reddish background will appear. \
The previous background will be restored after a valid pattern \
is reestablished. \
Some patterns cannot be typed without this happening. For example, \
when entering 'b[aeiou]d' as a pattern, it will be recognized as \
invalid as soon as '[' is typed. After ']' is typed, though, \
pattern validity will be regained. \
" ) ,

('Example Queries',
 "In the examples below, the queries are presented in the form:\
\n\n\
    <content terms>  ;  <path terms>  ;  <date terms>  ;  <file type>\
\n\n\
which is the order of the fields shown at the top of the client's window. \
\n\n\
  meeting january  ;  ;  ; \
\n\n\
    This query finds files whose contents include the words 'meeting' and \
'january'. \
\n\n\
  ;  pgsql .conf  ;  ; \
\n\n\
    This query finds all files whose path names contain the word 'pgsql' \
somewhere within the name and end with the extension '.conf'. \
\n\n\
  visit hawaii january  ;  ;  2004/8  ;  email/maildir \
\n\n\
    This query finds e-mail messages stored in 'maildir' format that \
mention 'visit', 'hawaii' and 'january'. Only those dated from \
August 1, 2004 to the present are included. \
\n\n\
  symbolic algebra  ;  user manual  ;  2001 2002/6/15  ;  doc/pdf \
\n\n\
    Using all four types of search terms, this query seeks PDF files dated \
from January 1, 2001 up to June 15, 2002 (15th is not included). The path \
names should contain the words 'user' and 'manual' while the file \
contents should include the words 'symbolic' and 'algebra'. \
" ) ,

('Indexed File Statistics' ,
 "Menu items View > Word Statistics (Path) and \
View > Word Statistics (Content) show histograms and summary \
statistics about the full-text indexes. This data is updated at the \
end of every 'full' indexing job. Updates are skipped during 'quick' \
indexing. Examining these statistics periodically will give you a \
sense of how the database is growing. Sampling the tail of the \
distribution can show if unexpected or undesirable 'words' are \
being recognized during indexing. This can help guide the development \
of profiles by identifying file categories that should be avoided. \
" ) ,

('User Preferences',
 "A small set of user preferences is maintained by the client. \
You may edit these settings as desired, within the constraints \
stipulated in the preferences file. Most preference changes will not take \
effect until the next time the client is started. \
" ) ,

('Command Line Queries',
 "Besides using the client interface, you may also invoke Pathena queries \
from the command line using the \
script 'ptfind', which was copied to your '~/bin' directory during \
installation. Type 'ptfind -h' to display usage information. \
\n\n\
Examples: \
\n\n\
    ptfind mars venus\n\
    ptfind 'saturn; ; 2004; doc/pdf'\n\
\n\
Results are returned through standard output in the form of matching \
path names, one per line. This allows the usual shell command features \
to be combined with Pathena search.  For example, using Pathena as a \
front-end to grep is a useful combination: \
\n\n\
    grep MAX_SIZE $(ptfind '; archives ; 1995 2000 ; source/c') \
\n\n\
Applications can use ptfind as a modest API that implements limited \
programmatic access to the database. \
" ) ,

('Pathena-Managed Files',
 "When the Pathena application is first installed, a dedicated user \
directory having path '~/.pathena/' is created to store client, \
server and database files. If you use the built-in editors for \
profiles and user preferences, there should be \
little reason to be concerned with these files.  Log files in \
subdirectory 'log' might be of interest on occasion. \
\n\n\
If you prefer to use a text editor to maintain profiles and \
user preferences, then you will be manipulating files in \
~/.pathena/profiles and ~/.pathena/config. For profiles, \
copy the file profile_template to a name of your choice. \
For config files, you may edit preferences.conf and source_custom.py \
directly because these are working copies created for you by the client. \
\n\n\
With the current layout of the Pathena distribution, when a new \
version becomes available, you can simply untar it on top of your current \
installation. This will overwrite the source files but leave other files \
intact. In particular, your profiles, preferences and database files \
should remain undisturbed. \
" ) ,

('Problem Reports',
 "If you experience a problem with Pathena, you are \
encouraged to send a report to the development team... \
\n\n\
... (procedure and location to be determined). \
" ) ,

#  "NOTE: the following feature is not yet implemented.\
# \n\n\
# If you experience a problem with Pathena, you may \
# send a report to the development team directly \
# from the client interface. \
# The menu item Submit Problem Report initiates this feature by displaying \
# a special window. It will also \
# be triggered automatically if the client detects an internal error.  \
# In either case a log file is prepared and stored in \
# the subdirectory '~/.pathena/log/'. The user is encouraged to type \
# a description of the problem into the upload form. \
# " ) ,

('Prototype Limitations',
 "Users should be aware of certain limitations of the current Pathena \
prototype. Many of these will be addressed in future releases. \
\n\n\
 - Some of the utilities for converting encoded files to text give poor \
results at times. PostScript is converted using ps2ascii, which can \
fail, work too slowly, or produce strange output. Converters for the \
Open Office file types are only crude demo versions. \
\n\n\
 - Converters for the markup types do minimal processing. HTML tags are \
stripped by Tsearch2 during indexing. The XML converter tries to do the \
same. Commands in TeX files are stripped as well. Displayed versions \
of these files, though, show the markup intact. \
\n\n\
 - Indexing speed for path names is adequate, typically in the range \
of 100-200 files per second. Content indexing speed will be only one \
tenth of this for a typical mix of file types. Document types such as \
PDF and especially PostScript tend to index rather slowly. \
Inherently textual material such as scripts or source files will \
index more quickly. \
\n\n\
 - Currently there is no way to remove indexed content or paths from \
the database, except for files that no longer exist, which are culled \
during full indexing runs. If many undesirable files have been \
indexed, they can be removed only by clearing the database and then \
reindexing your files using more restrictive profiles. \
\n\n\
 - To keep out less useful text that might clog the indexes, \
certain 'words' are filtered during the indexing process. Numbers and \
purely numeric file names are omitted. Hexadecimal constants in \
source files (strings of the form '0xnnnn') are filtered. Other \
categories might be added later. On occasion, searching for such \
strings might be desirable, but overall they were judged to do more harm \
than good. \
\n\n\
 - Searching for phrases is not yet supported. \
\n\n\
 - Tsearch2 uses a hashing technique for indexing and search that works well \
but can occasionally give approximate results. It might include items \
that don't satisfy the query. This effect creeps in when the number of \
distinct words grows large. \
\n\n\
 - Because of the hashing-induced inaccuracies and several other factors, \
when Pathena fails to find a file, you should not assume that it doesn't \
exist. Instead, you should regard Pathena's service as a helpful aid \
whose accuracy is high but less than 100%. \
\n\n\
 - Several actions should be avoided with the current prototype: \
running multiple client instances, running multiple indexing jobs \
simultaneously, indexing directories of file-like objects \
(e.g., /dev, /proc). \
\n\n\
" ) ,

('Usage Suggestions',
 "Here are a few tips for making the most productive use of Pathena. \
\n\n\
 - It is best to start with small profiles and \
gradually build up to larger subsets of your file system. After you \
gain a sense of which of your files can be indexed effectively, \
you will know how to construct useful profiles. \
\n\n\
 - Long files are truncated to an initial prefix before indexing. By \
default, only the first 100K bytes of converted text are retained. The \
rationale is that truncation saves time and space, and \
nearly all words needed for searching should appear within this prefix. \
If a different truncation size is desired, there \
is a user preference value that may be changed. \
\n\n\
 - Few preference values should need changing in practice. Font sizes \
might need some attention if any text displays are illegible. \
\n\n\
 - Queries that contain only negative search terms \
(in either Text or Path position) will run slowly because most paths \
in the database will match. \
\n\n\
 - Path name indexing should work well on most or all of the files in your \
home directory. Up to 200,000 path names should be indexable before you \
see performance degradation. \
\n\n\
 - Content indexing should be performed more judiciously. Too many words \
in the index will overburden the search engine and produce \
noticeable slowdowns. \
The number of indexable words varies widely by file type. Indexing more \
than 50,000 files is likely to impact performance. \
\n\n\
 - It helps to be alert for directories that can be excluded from \
content indexing; omitting them will keep the database lean. \
For example, directories saving backups, log files, \
temporary files, or software distributions \
don't normally need to be indexed. \
\n\n\
 - According to Tsearch2's designers, you should be mindful of the \
number of distinct words in a Tsearch2 index \
(Pathena keeps two: path words and content words). \
The useful upper limit is said to be in the range of 100-200K words. \
Pathena therefore cannot scale to large file systems in its current \
form. Future improvements in Tsearch2 should allow \
capacity and performance growth. \
\n\n\
 - For users who wish to extend Pathena to handle other file types, see \
the templates in ~/.pathena/types. Examine some modules (Python files) \
in that directory for relevant examples. \
Simply adding a new file-type module will \
cause it to be discovered at load time. \
\n\n\
" ) ,

('To Be Implemented',
 "Here is a partial list of desired future improvements. \
\n\n\
    - More converters to handle common file types \n\
    - Better converters for existing types \n\
    - Phrase indexing and searching \n\
    - Internationalization support \n\
    - Deleting unwanted files from the database \n\
    - Indexing triggered by file system events \n\
    - Higher indexing speeds \n\
    - Larger word and file capacities \n\
    - API for common operations \n\
" ) ,

)  # end of HelpItems
