
# This module implements various Tkinter widget utilities to make
# widget creation more convenient and more uniform.
#
##### This module still needs some scrubbing and generalizing.
#

from client_globals import *
import types

# ------------

class ThinButton(Button):
    def __init__(self, parent, min_width=8, **options):
        Button.__init__(self, parent, pady=1, **options)
        if 'width' not in options:
            self['width'] = max(min_width, len(self['text']))

class MediumButton(Button):
    def __init__(self, parent, min_width=8, **options):
        Button.__init__(self, parent, pady=3, **options)
        if 'width' not in options:
            self['width'] = max(min_width, len(self['text']))

class FullButton(Button):
    def __init__(self, parent, min_width=8, **options):
        Button.__init__(self, parent, **options)
        if 'width' not in options:
            self['width'] = max(min_width, len(self['text']))


# Construct a scrolled text widget.  Returns containing frame, text widget.

def scrolled_text_widget(parent, width=40, height=24, barwidth=10,
                         horiz_scroll=0, font=text_font, tab_stops=(),
                         **options):
    fr = Frame(parent)
    if 'wrap' not in options:
        if horiz_scroll: options['wrap'] = NONE # wrap_attr = NONE
        else:            options['wrap'] = WORD # wrap_attr = WORD
    txt = Text(fr, padx=5, pady=5, font=font, # wrap=wrap_attr,
               width=width, height=height, highlightthickness=0,
               background=colors['text_bg_color'], tabs=tab_stops, **options)
    scroll = Scrollbar(fr, command=txt.yview, width=barwidth)
    txt.configure(yscrollcommand=scroll.set)
#    fr.bind('<Enter>', lambda event: fr.focus_set())
    txt.grid(row=0, column=0, sticky='news')
    fr.rowconfigure(0, weight=1, minsize=0)
    scroll.grid(row=0, column=1, sticky='ns')
    fr.columnconfigure(0, weight=1, minsize=0)
    fr.columnconfigure(1, weight=0, minsize=0)
    if horiz_scroll:
        hscroll = Scrollbar(fr, command=txt.xview, orient=HORIZONTAL,
                            width=barwidth)
        txt.configure(xscrollcommand=hscroll.set)
        hscroll.pack(fill=X)  # side=LEFT, fill=Y)
        hscroll.grid(row=1, column=0, sticky='ew')
        fr.rowconfigure(1, weight=0, minsize=0)
    def enter_action(event):
       root.last_focus = root.focus_lastfor()
       txt.focus_set()
#    fr.bind('<Enter>', enter_action)
#    fr.bind('<Leave>', lambda event: root.last_focus.focus_set())
    return fr, txt

# Construct a scrolled listbox widget.  Returns containing frame, listbox.

def scrolled_listbox(parent, width=30, height=12, barwidth=10,
                     horiz_scroll=0, selectmode=SINGLE, font=text_font):
    fr = Frame(parent)
    listbox = Listbox(fr, selectmode=selectmode, font=font,
                      exportselection=0, background=colors['text_bg_color'],
                      height=height, width=width, highlightthickness=0)
    scrollbar = Scrollbar(fr, command=listbox.yview,
                          width=barwidth, elementborderwidth=2,
                          highlightthickness=0) # , bd=1, relief=SUNKEN)
    listbox.configure(yscrollcommand=scrollbar.set)
    listbox.grid(row=0, column=0, sticky='news')
    fr.rowconfigure(0, weight=1, minsize=0)
#        listbox.bind('<Enter>', lambda event: event.widget.focus_force())
    scrollbar.grid(row=0, column=1, sticky='ns')
    fr.columnconfigure(0, weight=1, minsize=0)
    fr.columnconfigure(1, weight=0, minsize=0)
    if horiz_scroll:
        hscroll = Scrollbar(fr, command=listbox.xview, orient=HORIZONTAL,
                            width=barwidth)
        listbox.configure(xscrollcommand=hscroll.set)
        hscroll.pack(fill=X)  # side=LEFT, fill=Y)
        hscroll.grid(row=1, column=0, sticky='ew')
        fr.rowconfigure(1, weight=0, minsize=0)

    def enter_action(event):
       root.last_focus = root.focus_lastfor()
       listbox.focus_set()
#    fr.bind('<Enter>', enter_action)
#    fr.bind('<Leave>', lambda event: root.last_focus.focus_set())
    return fr, listbox

# TODO: enable focus for text and listboxes but suppress highlight flashing.


# Construct a top-level window to display text with scrollbars.  Accept
# a procedure for adding buttons (or other widgets) at the bottom.  The
# procedure is passed the parent frame.  The text widget object is returned
# for later use to insert text.  The toplevel frame is optionally returned.

def scrolled_text_display(title='Pathena Client Text Display',
                          intro=None, aspect=2000, button_proc=null_proc,
                          font=text_font, horiz_scroll=0, barwidth=10,
                          width=70, height=24, geometry='-0+0',
                          tab_stops=(), return_frame=0, enter_focus=1):
    fr = Toplevel()
    fr.geometry(newGeometry=geometry)
    if intro: Message(fr, aspect=aspect, font=text_font,
                      text=intro).pack(padx=10, pady=10)
    txt_fr = Frame(fr)
    txt = Text(txt_fr, padx=10, pady=5, wrap=WORD, highlightthickness=0,
               width=width, height=height, font=font,
               background=colors['text_bg_color'], tabs=tab_stops)
    scroll = Scrollbar(txt_fr, command=txt.yview, width=barwidth)
    txt.configure(yscrollcommand=scroll.set)
    if enter_focus: fr.bind('<Enter>', lambda event: txt.focus_set())
    txt.pack(side=LEFT, fill=BOTH, expand=YES)
    scroll.pack(side=LEFT, fill=Y)
    if horiz_scroll:
        hscroll = Scrollbar(txt_fr, command=txt.xview,
                            width=barwidth, orient=HORIZONTAL)
        txt.configure(xscrollcommand=hscroll.set)
        hscroll.pack(fill=X)  # side=LEFT, fill=Y)
    txt_fr.pack(padx=5, pady=5, fill=BOTH, expand=YES)
    button_proc(fr)
    fr.title(string=title)
    if return_frame: return fr, txt
    else:            return txt

# Construct a top-level window to display a listbox with scrollbars.  Accept
# a procedure for adding buttons (or other widgets) at the bottom.  The
# procedure is passed the parent frame.  The listbox widget object is returned
# for later use by the caller.  The toplevel frame is optionally returned.

def scrolled_listbox_display(title='Pathena Client Listbox Display',
                             width=30, height=12, barwidth=10,
                             selectmode=SINGLE, geometry='-0+0',
                             intro=None, aspect=2000, button_proc=null_proc,
                             font=list_font,
                             return_frame=0, enter_focus=1):
    fr = Toplevel()
    fr.geometry(newGeometry=geometry)
    if intro: Message(fr, aspect=aspect, font=text_font,
                      text=intro).pack(padx=10, pady=10)
    listbox_fr = Frame(fr)
    listbox = Listbox(listbox_fr, selectmode=selectmode, bd=1, relief=SUNKEN,
                      exportselection=0, background=colors['text_bg_color'],
                      height=height, width=width, highlightthickness=0)
    scrollbar = Scrollbar(listbox_fr, command=listbox.yview,
                          width=barwidth, elementborderwidth=2,
                          highlightthickness=0, bd=1, relief=SUNKEN)
    listbox.configure(yscrollcommand=scrollbar.set)
    listbox.pack(side=LEFT, fill=BOTH, expand=YES)   #, padx=4, pady=4)
    scrollbar.pack(side=LEFT, fill=Y, expand = YES)  #, pady=4)
    if enter_focus: fr.bind('<Enter>', lambda event: listbox.focus_set())
    listbox_fr.pack(padx=5, pady=10)
    button_proc(fr)
    fr.title(string=title)
    if return_frame: return fr, listbox
    else:            return listbox


# Create a scrolled text widget that will be populated by rows of
# tabular information.  Accept a list of header labels to display
# above the text widget.

def scrolled_table_text(parent, width=40, height=24, tab_stops=(),
                        header_labels=()):
    fr = Frame(parent)
    header = Frame(fr)
    for h in header_labels:
        Label(header, text=h).pack(side=LEFT)
    header.pack(fill=X, pady=5)
    stops = tuple([ t * fixed_text_pixels for t in tab_stops ])
    table_fr, table_text = \
              scrolled_text_widget(fr, width=width, height=height,
                                   tab_stops=tab_stops, font=small_fixed_font)
    table_fr.pack(padx=5, pady=5)
    return fr, table_fr, table_text
            

# Build a table using grid geometry manager.
# Accepts a list of functions for creating the widgets in each column
# of the rows.  Each function has signature [parent, arg -> widget].
# If a header row is needed, just supply it as the first row.
# The initialization args to these functions are supplied
# as a list of row lists (header is first row if used).
# Each row list item is an arg supplied to the
# corresponding column widget function.
# Tuples may be used instead of lists, of course.

# An optional anchor list may be supplied to indicate the positioning
# of widgets within their cells.  Left justification is the default.
# If a value item has the form of a tuple, its second element is
# considered an optional anchor that overrides the column anchor.
# Optional row and column weight lists may be supplied to indicate the
# expandability of rows and columns.  The default is 1 (expandable).
# Returns the outer frame (unpacked) and the table widgets as a list of rows,
# each item of which is a list of column widgets.

def build_table(parent, col_functions, init_values, anchors=(),
                col_weights=(), row_weights=(), padx=0, pady=0):
    fr = Frame(parent)
    num_rows = len(init_values)
    num_cols = len(col_functions)
    if not anchors: anchors = [ W for f in col_functions ]
    if not col_weights: col_weights = [ 1 for f in col_functions ]
    if not row_weights: row_weights = [ 1 for v in init_values ]
    col_indices = range(num_cols)
    row_widgets = []
    def build_row(row, init_vals):
        col_widgets = []
        for c, wfn, val, anchor in \
                zip(col_indices, col_functions, init_vals, anchors):
            if type(val) == types.TupleType: val, anchor = val
            widget = wfn(fr, val)
            col_widgets.append(widget)
            widget.grid(row=row, column=c, sticky=anchor, padx=padx, pady=pady)
        row_widgets.append(col_widgets)
    for row, init_vals, wt in zip(range(num_rows), init_values, row_weights):
        build_row(row, init_vals)
        fr.rowconfigure(row, weight=wt, minsize=0)
    for col, wt in zip(col_indices, col_weights):
        fr.columnconfigure(col, weight=wt, minsize=0)
    return fr, row_widgets


# Pop up a toplevel window.  Function display_fn takes a parent frame
# as first argument and displays its widgets within that frame.  A list
# of other arguments to the display function is supplied by the caller.

def pop_up_toplevel_window(display_fn, args, title,
                           corner=0, x_offset=0, y_offset=0,
                           top_margin=10, bottom_margin=10):
    fr = Toplevel()
    fr.withdraw()  # unmap window while building to prevent flash ###
    fr.geometry(newGeometry='-0+0')  # temp position
    fr.title(string=title)
    Frame(fr).pack(pady=top_margin)
    display_fn(fr, *args)
    Frame(fr).pack(pady=bottom_margin)
    rx,ry = root.winfo_rootx(), root.winfo_rooty()
    if corner:
        fr.deiconify()  # remap window  ###
        return          # leave in upper right corner
    fr.update()
    x = (root.winfo_width() - fr.winfo_reqwidth()) // 2
    y = (root.winfo_height() - fr.winfo_reqheight()) // 2
    # move window to final position
    fr.geometry(newGeometry='+%d+%d'%(rx + x + x_offset, ry + y + y_offset))
    fr.deiconify()  # remap window  ###
    return fr


# For right-click popup menus.  Supply a list of items: [(label, proc)],...].
# Insert None in the list to create a separator.  Use (label, list) to
# indicate a cascading submenu.  If any items have form (label, proc, tag),
# a dictionary is created and returned that maps from tags to list
# item numbers.

def add_popup_menu(parent, menu_items):
    popup = Menu(parent, tearoff=0)
    tag_dict = {}
    for item in menu_items:
        if item and callable(item[1]):
            popup.add_command(label=item[0],
                              command=EventHandler('Popup menu: %s' %
                                                   item[1].__name__, item[1]))
        elif item:
            popup.add_cascade(label=item[0],
                              menu=add_popup_menu(popup, item[1]))
        else:
            popup.add_separator()
    for i in range(len(menu_items)):
        if menu_items[i] and len(menu_items[i]) > 2:
            tag_dict[menu_items[i][2]] = i
    if tag_dict: return popup, tag_dict
    else:        return popup


_dummy_entry = Entry()   # don't display; for export selection only

# Add a row of buttons given a list of descriptor tuples
# (name, command [, watch, disabled]).
# Returns frame (unpacked), list of button widgets

def button_row(parent, button_list, row_descriptor='', padx=5, size=1):
    button_fr = Frame(parent)
    widgets = []
    for button in button_list:
        event_name = ' '.join((row_descriptor, button[0])).strip()
        if len(button) > 2:
            command = EventHandler(event_name, button[1], button[2])
        else:
            command = EventHandler(event_name, button[1])
        button_class = (ThinButton, MediumButton, FullButton)[size]
        w = button_class(button_fr, text=button[0], command=command)
        if len(button) > 3 and button[3]:
            w.configure(state=DISABLED)
        w.pack(side=LEFT, padx=padx)
        widgets.append(w)
    return button_fr, widgets

def copy_text_clipboard(txt):
    _dummy_entry.delete(0, END)
    _dummy_entry.insert(END, txt)
    _dummy_entry.selection_range(0, END)        # for Unix/Linux
#    ent.clipboard_clear()
#    ent.clipboard_append(ent.get())    # needed for Windows
#    Button(fr, text='Copy File Name to Clipboard',
#           command=lambda : copy_entry_clipboard(file_ent)).pack()

def displayMessage(text, simple):
   if simple>0:
      messageWindow = SimpleDialog(master=mainwindow, text=text,
                                   buttons=['OK', 'CANCEL'], cancel=1)
   else:
      messageWindow = SimpleDialog(master=mainwindow, text=text,
                                   buttons=['Close'], cancel=0)
   messageWindow.go()
   return messageWindow.num


def view_log_file(title, file_name, init_pos=0.0):
    count = [None]      # mutable variable
    def add_buttons(fr):
        but_fr = Frame(fr)
        count[0] = Label(but_fr, text='')
        count[0].pack(side=LEFT, padx=20)
        button_row(but_fr, (('Redisplay', lambda : show_all(), but_fr),
                            ('Close',     lambda : fr.destroy(), 0))
                   )[0].pack(side=LEFT, padx=10, pady=2)
        but_fr.pack()
    txt = scrolled_text_display(title=title, width=72, barwidth=11,
                                intro='File: %s' % file_name,
                                button_proc=add_buttons)
    def show_all():
        txt.delete(1.0, END)
        lines = read_text_file(file_name, 1)
        for v in lines: txt.insert(END, v)
        count[0].config(text='%d lines in log file' % len(lines))
        txt.yview_moveto(1.0)
    show_all()

def view_text_file(title, file_name, init_pos=0.0):
    def add_buttons(fr):
        button_row(fr, (('Close', lambda : fr.destroy(), 0)))[0].pack()
    txt = scrolled_text_display(title=title, width=72, barwidth=11,
                                intro='File: %s' % file_name,
                                button_proc=add_buttons)
