
# Initialize the PostgreSQL-resident server functions for the Pathena
# indexing process.  This module is not needed (and should not be imported)
# by the client or other components.

import sys, os, time

#=================================


import string, glob, shutil
from operator import add, and_
from StringIO import StringIO
from traceback import print_exc

from app_globals        import *
from platform_utilities import *
from file_type_attribs  import *

indexed_file_max_size = 100000000   # limit content indexing to smaller files

_temp_file = new_temp_file_name()
_temp_file_decomp = new_temp_file_name()



# Index file content using stored procedures.  Ignore if current mod time
# in database is still current.  Return count of files indexed (0 or 1).

def index_file_text(path, use_path, file_type, decompress_cmd=''):
    try:
        cur_mod_time = os.path.getmtime(path)
    except:
        return '1'   # file now gone
    qpath = quote_quote(path)
    db_attribs = \
        _GD["execute_query"](
            "select mod_time, file_type, length(content_vector) as con_len "
            "from file_descriptors where abs_path = '%s'" % qpath)
                          
    # no need to index if mod_time is current; will index if current
    # record has no tsvector
    if db_attribs and \
       db_attribs[0]["mod_time"] == cur_mod_time and \
       db_attribs[0]["file_type"] == file_type and \
       db_attribs[0]["con_len"] != None:
        # if previously path only => add content
        return '0'       # existing tuple is still valid

    type_class = file_type_class[file_type]
    if decompress_cmd: db_caching = 2
    else:              db_caching = type_class.cache_conversion(path)
    try:
        if decompress_cmd:
            conv_path = _temp_file_decomp
            make_shell_command('%s > %s' % (decompress_cmd, conv_path))(path)
        else:
            conv_path = path
        if db_caching == 1:
            # convert using cached content mode (3)
            cache_text = type_class.convert_file(conv_path, 3)
            text       = prepare_for_db(type_class.convert_cached_content(
                                            cache_text, 1))
            cache_text = prepare_for_db(cache_text)
        elif db_caching == 2:
            # convert using cached content mode (3), index mode (1) separately
            cache_text = prepare_for_db(type_class.convert_file(conv_path, 3))
            text       = prepare_for_db(type_class.convert_file(conv_path, 1))
        else:
            # convert using default indexing mode (1)
            text = prepare_for_db(type_class.convert_file(conv_path, 1))
        text = post_conversion(text, type_class.file_indexing_attrib)
        if db_caching:
            title    = prepare_for_db(
                           type_class.extract_title(conv_path, path))
            metadata = prepare_for_db(
                           type_class.extract_metadata(conv_path, path))
        try:    os.remove(_temp_file_decomp)
        except: pass
    except Exception, exc_obj:
        warning_msg = \
            '*** Error converting file %s :\nException: %s.' % \
            (path, ' ; '.join([ str(a) for a in exc_obj.args ]))
        try:    os.remove(_temp_file_decomp)
        except: pass
        return warning_msg
    if text == '*****':
        warning_msg = \
            '*** An error occurred while converting file %s to text.' % path
    else:
        warning_msg = ''

    if use_path: p_words = path_words(type_class.path_excerpt(path))
    else:        p_words = ''
    if len(text) >= max_indexed_len:
        text = trim_partial_word(text)
    if db_caching:
        query_text = "select " \
            "index_file_cache('%s','%s','%s','%s','%s','%s','%s','%s')" % \
            (qpath, p_words, file_type, title, metadata, cache_text,
             text, str(cur_mod_time))
        _GD["execute_query"](query_text)
    else:
        query_text = "select index_file('%s', '%s', '%s', '%s', '%s')" % \
                     (qpath, p_words, file_type, text, str(cur_mod_time))
        _GD["execute_query"](query_text)
    return warning_msg

def prepare_for_db(text):
    # replace NUL chars and single quotes
    text = text.replace("\x00", " ").replace("'", " ")   # "''")
#    check_for_unicode(path, text)
    return text

def trim_partial_word(text):
    last = len(text) - 1
    while last and text[last].isalnum():
        last -= 1
    return text[:last+1]

#def check_for_unicode(path, text):
#    for c in text:
#        if ord(c) > 127:
#            emit_log_entry(main_log_file, 'Contains unicode: %s' % path)
#            break


#=================================


def init_support_ops(GD):
    global _GD, PATHENA
    _GD = GD
    PATHENA = GD["PATHENA"]

    ### add exception recording and announcement
    try:
        for handler in glob.glob(os.path.join(PATHENA, 'types', '*.py')):
            __import__(os.path.splitext(os.path.basename(handler))[0])
    except:
        GD["emit_plpy_log_entry"](
            "Failed to import types module '%s'." % handler)

    GD["index_file_text"] = index_file_text

    GD["emit_plpy_log_entry"]('Indexing startup complete.')
