
# Initialize the PostgreSQL-resident server functions of Pathena.
# Load modules and set items in PlPython's global data dictionary GD.
# Procedure 'start_up' is called when a new connection is created and
# the backend initializes a new environment.  The following definitions
# remain in effect as long as the backend process is running.

import sys, os, time

from app_globals       import *
from logging_services  import *

import file_type_attribs

main_log_file =  open(os.path.join(log_dir, 'server_side'), 'a')

sys.stderr = main_log_file

def emit_plpy_log_entry(log_entry):
    emit_log_entry(main_log_file, log_entry)

def get_file_contents(path, nchars=-1):
    try:
        file = open(path)
        ftext = file.read(nchars)
        file.close()
        return ftext
    except Exception, exc_obj:     # file permission or other error
        msg = ' ; '.join([ str(a) for a in exc_obj.args ])
        try:
            file.close()
        except:
            pass
        emit_plpy_log_entry("Error reading file '%s':\n  %s" % (path, msg))
        return  ''

def make_execute_query(plpy_execute):
    def execute_query(query):
        try:
            return plpy_execute(query)
        except:
            from StringIO import StringIO
            from traceback import print_exc
            trace = StringIO()
            print_exc(1, trace)
            if len(query) > 500: tail = "\n< . . . remainder truncated>"
            else:                tail = ""
            emit_plpy_log_entry(trace.getvalue() +
                                "\n>>>>> Failed execution for query:\n" +
                                query[:500] + tail)
            return None
    return execute_query


#=================================


def start_up(GD):
    global _GD, PATHENA
    _GD = GD
    PATHENA = GD["PATHENA"]

    GD["emit_plpy_log_entry"] = emit_plpy_log_entry
    GD["get_file_contents"]   = get_file_contents
    GD["execute_query"]       = make_execute_query(GD["plpy.execute"])
    GD["file_type_attribs"]   = file_type_attribs

    emit_log_entry(main_log_file, 'Python server-side startup complete.')
