# Pathena file-type class definition

from file_type_attribs import *

# ============== Do not change anything above this line ==============


# ZIP archive file type.

species_name    = 'zip'

import re

# Output from an unzip listing looks like this:

# Archive:  /home/bld/tmp/app.zip
#   Length     Date   Time    Name
#  --------    ----   ----    ----
#         0  05-04-04 23:26   app/
#     30249  04-29-04 16:26   app/main.py
#      . . .   
#      1504  01-30-04 21:23   app/License
#  --------                   -------
#    230822                   32 files

# We need to extract only the path name parts for indexing.

file_line_pattern = re.compile(r"\d{2,2}:\d{2,2}")

def strip_to_path_names(content):
    return ''.join([ ' '.join(filter(None, line.split(' '))[3:])
                     for line in content.splitlines(1)
                     if file_line_pattern.search(line) ])

converter_exists, file_converter = \
    make_converter_command('unzip', 'unzip -l %s')

#def file_converter():
#    return make_shell_command('unzip -l %s')


class Regular_Type(Archive_Genus):

    def __init__(self, name):
        Archive_Genus.__init__(self, name)

    file_extensions = ('zip',)

    commands_needed = ('unzip',)

    def convert_file(self, path, usage=0):
        if converter_exists and usage in (1, 2):     # indexing and headlines
            return strip_to_path_names(
                       protected_convert_file(file_converter, path, usage))
        elif converter_exists:      # display and cache
            return protected_convert_file(file_converter, path, usage)
        if usage == 0:              # display
            return file_converter(path)
        else:
            return ''

    
    def convert_cached_content(self, content, usage=0):
        if usage == 0:
            return content
        else:
            # retain only path names for indexing, headlines
            return strip_to_path_names(content)


# ============== Do not change anything below this line ==============

register_type_class(Regular_Type(species_name))
