# Pathena file-type class definition

from file_type_attribs import *

# ============== Do not change anything above this line ==============


# PDF document file type.


species_name    = 'pdf'

# convert PDF to text, suppressing Latin1, UTF-8 chars:
# pdftotext -enc ASCII7 %s -

converter_exists, file_converter = \
    make_converter_command('pdftotext',
                           truncated_spawn('pdftotext', '-enc ASCII7', '-',
                                           command_timeout))

# alt: retain original character sets
# file_converter = make_shell_command('pdftotext %s -')

title_extractor = make_shell_command('pdfinfo %s | grep Title:')

metadata_extractor = make_shell_command('pdfinfo %s')


class Regular_Type(Document_Genus):

    def __init__(self, name):
        Document_Genus.__init__(self, name)

    file_extensions = ('pdf',)

    commands_needed = ('pdftotext', 'pdfinfo')
    
    external_viewers = (('xpdf', 'xpdf "%s"'),
                        ('gv', 'gv "%s"'),
                        ('kghostview', 'kghostview "%s"'),
                        ('gpdf', 'gpdf "%s"'))
    
#    external_editors = None


    if converter_exists:
        def convert_file(self, path, usage=0):
            return protected_convert_file(file_converter, path, usage)
    else:
        def convert_file(self, path, usage=0):
            if usage == 0:      # display
                return file_converter(path)
            else:
                return ''
    
    def extract_title(self, path, orig_path=None):
        return ' '.join(title_extractor(path).split()[1:])

    def extract_metadata(self, path, orig_path=None):
	 return metadata_extractor(path)


# ============== Do not change anything below this line ==============

register_type_class(Regular_Type(species_name))
