# Pathena file-type class definition

from file_type_attribs import *

# ============== Do not change anything above this line ==============


# Special file type class for 'maildir' files.

# E-mail messages in this format are stored one message per file.
# Any e-mail client following the maildir conventions will be supported.
# A typical path name for a maildir file is
# '~/Mail/inbox/cur/1093725412.1681.AmDE:2,S'.


# Additional module imports:

import os
import email_utils


# Provide the special file name as a string (required).  This name
# will serve as the file type.

species_name = 'maildir'

path_split = os.path.split


class Special_Type(Email_Genus):

    def __init__(self, name):
        Email_Genus.__init__(self, name)

    phylum = 'special'
    
#    external_viewers = None
#    external_editors = None
    
    def convert_file(self, path, usage=0):
        return email_utils.prune_message(path)
    
    def extract_title(self, path, orig_path=None):
        return email_utils.extract_subject(path)
    
#    def extract_metadata(self, path, orig_path=None):
#	 return ''

    # Permanent maildir files have parent directory 'cur' and
    # grandparents given by mailbox names.  Message file names
    # follow a convention, but there is no need to decode them
    # as we suppress these file names entirely.
    # Result is a pair (use_path, include_content).
    
    def recognize_file(self, path, file_name):
        include = os.path.isfile(path) and \
                  not file_name.startswith('.') and \
                  os.path.split(os.path.dirname(path))[1] == 'cur'
        return (1, include)

    # Path names have a form similar to this:
    #   /home/bld/Mail/inbox/cur/1110757235.12108.EJwoF:2,S
    # We drop the last two components: cur/<file>.
    
    def path_excerpt(self, path):
        return path_split(path_split(path)[0])[0]



# ============== Do not change anything below this line ==============

register_type_class(Special_Type(species_name))
