# E-mail messages in MIME format are pruned to strip off uninteresting
# header fields, file attachments, and other unwanted parts.  A short
# annotation replaces each deleted attachment.

import email, StringIO

retained_keys = ('from', 'to', 'subject', 'date', 'cc', 'bcc', 'content-type')

# Return a pruned message as a single string.

def prune_message(file):
    mfile = open(file)
    m = email.message_from_file(mfile)
    mfile.close()
    attach = []
    if m.is_multipart():
        for a in m.get_payload()[1:] :
            attach.append('\n===========================================\n'
                          'Attachment   Type: %s   File: %s\n'
                          '< content omitted >\n'
                          % (a.get_type(), a.get_filename()))
        m.set_payload(m.get_payload()[0]) # strip off attachments
    purged_keys = [ k for k in m.keys() if k.lower() not in retained_keys ]
    for k in purged_keys: del m[k]        # remove header clutter
    body = purge_base64(m.as_string())[:100000]
    return body + ''.join(attach)         # purge, truncate long messages

# Try to detect presence of nonstandard base64 lines that escaped
# pruning using the email module.  If two consecutive lines begin
# with 'M' and are followed by exactly 60 more characters (then \n),
# declare them part of a base64 encoding stream and drop them from
# the output string.

def purge_base64(str):
    result = StringIO.StringIO()
    num_matches = 0
    prev_line = ''
    for line in StringIO.StringIO(str):
        if len(line) == 62 and line[0] == 'M':
            num_matches += 1
            prev_line = line
        else:
            if num_matches == 1:
                result.write(prev_line)
            result.write(line)
            num_matches = 0
    return result.getvalue()


# Extract subject field for use as a file title.
# Find a way to avoid parsing message twice when both this function
# and prune_message are evaluated.

def extract_subject(file):
    mfile = open(file)
    m = email.message_from_file(mfile)
    mfile.close()
    return m['Subject']
