# Pathena file-type class definition and utilities

from file_type_attribs import *


# Open Office common definitions

import os, re
import zipfile

stylesheet_file = os.path.join(types_dir, 'null.xslt')

xslt_cmd = make_shell_command('xsltproc --novalid "%s" %%s' % stylesheet_file)
    
def extract_subfile(path, subfile):
    return zipfile.ZipFile(path).read(subfile)

# conversion provided (in part) by xsltproc if it exists:
    
def xml_subfile_converter(subfile):
    def extract_xml_text(path):
        temp_file = new_temp_file_name()
        content = open(temp_file, 'w')
        # insert newline at end of each XML element to prevent run-on text
        # and improve readability
        content.write(extract_subfile(path, subfile).replace('</', '\n</'))
        content.close()
        text = xslt_cmd(temp_file)
        os.remove(temp_file)
        return purge_extra_blank_lines(text)
    return extract_xml_text

converter_exists, file_converter = \
    make_converter_command('xsltproc', xml_subfile_converter('content.xml'))

######## add truncated_spawn???

title_pattern = re.compile(r'<dc:title>(.*)</dc:title>')

def title_extractor(path):
    metadata = extract_subfile(path, 'meta.xml')
    match = title_pattern.search(metadata)
    if match: return match.group(1)
    else:     return ''

if converter_exists:
    metadata_extractor = xml_subfile_converter('meta.xml')
else:
    metadata_extractor = null_string_fn


# Introduce a common file type class usable by all OOffice file types

class OOffice_Type(Document_Genus):

    def __init__(self, name):
        Document_Genus.__init__(self, name)

#    file_extensions = ('sx?',)

#    external_viewers = (('oo???', 'oo??? "%s"'), )

    commands_needed = ('xsltproc',)
    
    def convert_file(self, path, usage=0):
        if converter_exists:
            return protected_convert_file(file_converter, path, usage)
        elif usage == 0:
            return file_converter(path)
        else:
            return ''
    
    def extract_title(self, path, orig_path=None):
        return title_extractor(path)

    def extract_metadata(self, path, orig_path=None):
        return metadata_extractor(path)
